#!/usr/local/bin/wish40

#####################################################################
# FILE:    compare_segmentation.tcl
# AUTHOR:  Giovanni Flammia 
#
# NOTE: If you dont specify otherwise in the command line arguments, this program
#       assumes that the segment tags are called <Segment_Number>...</Segment_Number>
#       (for example, when called from "View Other")
#
#         (N)[b] Copyright (c) 1996
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
#####################################################################

if [catch {set tkver $tk_version}] {
    puts "Sorry. The version of Tk you're running is too old for Nb."
    puts "Nb runs under Tk version 4.0 or later and Tcl version 7.4 or later."
    puts "You can download the latest version of Tcl/Tk at: http://www.sunlabs.com:80/research/tcl/"
    exit 
}


if [catch {set tclver [info tclversion]}] {
    puts "Sorry. The version of Tcl you're running is too old for Nb."
    puts "Nb runs under Tcl version 7.4 or later and Tk version 4.0 or later."
    puts "You can download the latest version of Tcl/Tk at: http://www.sunlabs.com:80/research/tcl/"
    exit 
}


if {$tkver < 4.0} {
    puts "Sorry. The version of Tk you're running is: $tkver."
    puts "Nb runs under Tk version 4.0 or later and Tcl version 7.4 or later."
    puts "You can download the latest version of Tcl/Tk at: http://www.sunlabs.com:80/research/tcl/"
    exit 
}


if {$tclver < 7.4} {
    puts "Sorry. The version of Tcl you're running is: $tclver."
    puts "Nb runs under Tk version 4.0 or later and Tcl version 7.4 or later."
    puts "You can download the latest version of Tcl/Tk at: http://www.sunlabs.com:80/research/tcl/"
    exit 
}

### puts "Running Tcl version $tclver and Tk version $tkver"

# Process input arguments:
if { $argc < 1 } {
  puts "Usage: wish compare_segmentations.tcl \[ file1 file2 ... \]"
  # Under windows, running from the wish console without arguments make it crash on exit, so...
  set argc 1
  set argv [list default]
  catch {set argv [lsort [glob -nocomplain -- *nb]]}
  set argc [llength $argv]
}




proc CustomBind {} {}

set colorTable(TopLevel) white
set inputFiles           $argv 
set segmentTags          [list]
set segmentType "Segment"


# Name the window and global variables:
wm title . "Nb Segmentations"
wm iconname . "Nb Segmentations"


set bg1                "Grey"
set fg1                "black"
set text_bg            "white"
set text_fg            "black"
set text2_bg           "Grey"
set text2_fg           "Black"
set scroll_bg          "Black"
set listbox_bg         "Grey"
set listbox_fg         "Black"
set annotated_text_lines 30
set textArea .top
set font1 fixed

if {[info exists env(NB_FONT)]} {
  set font1 $env(NB_FONT)
} else {
  set font1  "lucidasanstypewriter-bold-12"
}

set font1 "-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1"

# define platform independent file manipulation functions: 
# native_to_unix, my_file_dirname, my_pwd, my_cd
proc native_to_unix {native} {

  switch -regexp -- $native {
    ^:$   {set this [pwd]}
    ^.$   {set this [pwd]}
    ^..$  {set this [file dirname [pwd]]}
    ^::$  {set this [file dirname [pwd]]}
    default {set this $native}
  }
  
  switch -regexp -- $this {
    ^[A-Za-z]\:[\/\\]
    { regsub -all -- {([\\]+)} $this "/" result }
    [:]+ 
    {
      if {[regexp {^[a-zA-Z]} $this match]} {
	set this "/$this"
      }
      regsub -all -- {([:]+)} $this "/" result
    }
    default {set result $this}
  }
  
  return $result
}


proc my_file_dirname {file_name} {
  return [native_to_unix [file dirname $file_name]]
}


proc my_pwd {} {
  return [native_to_unix [pwd]]
}


proc my_cd {directory_name} {
  cd [native_to_unix $directory_name]
}


# Begin Global variables:
set NB_SOURCE [my_file_dirname [info script]]

set rootname ""
set dirname [my_pwd]
regexp {^([-_a-zA-Z0-9]+\.)} [lindex $argv 0] match rootname
if {$argc >= 1} {
  set dirname [my_file_dirname [lindex $argv 0]]
  ###puts "dirname is: $dirname"
  ###puts "first file is: [lindex $argv 0]"
  regexp {^([-_a-zA-Z0-9]+\.)} [file tail [lindex $argv 0]] match rootname
}

if {[string compare $rootname ""] == 0} {
  regexp {^([-_a-zA-Z0-9]+\.)} [lindex $argv 0] match rootname
}

###puts "rootname is: \"$rootname\""

# List all the current idTable(Tags) under the cursor:
proc ListTagsUnderCursor {} {
  global idTable tagUnderCursor textArea
  set listId ""
  set spaces " "
  set dots "..."
  foreach oneTag [$textArea.text tag names "current"] {
    if {[info exists idTable($oneTag)]} {
      if {[regexp {( $idTable($oneTag))} $listId match value] == 1} {
   # set listId $listId 
      } else {
   if {[expr [string length $listId] < 30]} {
     set listId $listId$spaces$idTable($oneTag)
   } else {
     set listId $listId$dots
   }
      }
    }
  }
  set tagUnderCursor $listId
}


# Color palette for X terminals:
set palette [list honeydew3 salmon3 LightSeaGreen Orange PaleGreen PaleTurquoise LemonChiffon azure4 gold1 grey50 SeaGreen4 LightYellow4 PeachPuff4 MistyRose PowderBlue blue yellow red green black ]
set palette_size [llength $palette]
set other_palette [list]
proc SetSixtyFourColors {} {
  
  global other_palette
  
  set rgb(0,0) 127
  set rgb(1,0) 100
  set rgb(2,0) 127
  
  set rgb(0,1) 100
  set rgb(1,1) 127
  set rgb(2,1) 100
  
  set rgb(0,2) 127
  set rgb(1,2) 127
  set rgb(2,2) 100
  
  set max_num_colors 64
  set this 0
  
  for { set i 0 } { $i < $max_num_colors } { incr i } {
    
    for { set j 0 } { $j < 3 } { incr j } {
      if { $j != $this } {
	set rgb($j,$this) [expr $rgb($j,$this) + [expr $j * 10]]
      }
      set rgb($this,$this) [expr $rgb($this,$this) + 4]
    }
    
    if { $rgb(0,$this) < 255 } {
      if { $rgb(1,$this) < 255 } {
	if { $rgb(2,$this) < 255 } {
	  set this_color [format "#%2lx%2lx%2lx" $rgb(0,$this) $rgb(1,$this) $rgb(2,$this)]
	  ###puts "Adding color: $this_color"
	  lappend other_palette $this_color
	}
      }
    }
    
    incr this
    if { $this >= 3 } { set this 0 }
    
  }
}

SetSixtyFourColors
set other_size [llength $other_palette]

# Check whether there is a color coding scheme available
set current_dir $dirname
set colorfile   "nil"
set colorCode(0.0) 0
set colorIndex  0
set alignedFiles 0

catch {set colorfile [glob -nocomplain -- $current_dir/align/$rootname*color_codes]}
if {[string compare $colorfile "nil"] != 0} {
  if {[string compare $colorfile ""] != 0} {
    if {[file exists $colorfile]} {
      set file [open $colorfile r]
      ###puts "Reading color codes from $colorfile..."
      while {[gets $file line] >= 0} {
	if {[regexp {([0-9]+)\.[ ]*([0-9]+)[ ]+([0-9]+)} $line match x y c]} {
	  incr x
	  incr y
	  set alignedFiles 1
	  set c [expr $c % $palette_size]
	  set colorCode($x.$y) $c
	}
      }
      close $file
    } else {
      set other_palette $palette
      set other_size [llength $other_palette]
    }
  } else {
    set other_palette $palette
    set other_size [llength $other_palette]
  }
} else {
  set other_palette $palette
  set other_size [llength $other_palette]
}



set tagUnderCursor ""

proc WarningMessage { message } {
    puts $message
}

# Source files:
source $NB_SOURCE/file_functions.tcl
source $NB_SOURCE/output_functions.tcl

# Hold text and vertical scrollbar
frame .top -bg $bg1 

# Hold horizontal scrollbar and padx
frame .mid -bg $bg1   

# Hold dismiss and other buttons
frame .bot -bg $bg1

# Begin global variables:

# Hashtables for cascade-menu Tags:
# fieldTable(firstLevelTag) = list with currently used index for firstLevelTag
set fieldTable(TopLevel)  [list]

# choiceTable(Tag) = list with pull-down menu options for Tag
set choiceTable(TopLevel) [list]

# priority table for nested tags like: <TAG1> <TAG2> ... </TAG2> <TAG1>
set priorityTable(TopLevel) 0

# For each index, keep a list of tags in preorder:
set beginAtIndex(0.0) [list]
set endAtIndex(0.0) [list]

# Each tag has the format topLevelTag_number and is associated with
# a name in idTable (a string), a color in colorTable, and a range.
# Hastables for text Tags:
set listOfSgmlTags [list]
set idTable(TopLevel)     "TopLevel"

# Recently used color in fieldTable(Colors)
set colorIndex 0

# set rangeTable(TopLevel)  { "1.1" "1.1" }
set rangeTable(TopLevel) [list]

# End global variables
# Main window text widget: by default you can't write on it:
set textArea .top
text      $textArea.text   -relief sunken -font $font1 -bg $text_bg -fg $text_fg \
    -yscrollcommand "$textArea.yscroll set" \
    -xscrollcommand ".mid.xscroll set" \
    -exportselection 1 \
    -state normal \
    -height $annotated_text_lines -wrap none -setgrid 1 
scrollbar $textArea.yscroll  -orient vertical -command "$textArea.text yview" -relief raised -bg $scroll_bg
scrollbar .mid.xscroll  -orient horizontal -command "$textArea.text xview" -relief raised -bg $scroll_bg




# create padding based on size of scrollbars (p. 193 of Welch's book)
set pad [expr [$textArea.yscroll cget -width] + 2* \
        ([$textArea.yscroll cget -bd] + \
        [$textArea.yscroll cget -highlightthickness])]
frame .mid.pad -width $pad -height $pad -bg $bg1
button .bot.dismiss -text "Dismiss" -font $font1 -command exit -relief raised -bg $bg1 -fg $fg1 -padx 20 
frame .bot.pad -width $pad -height $pad -bg $bg1
label .bot.label -textvariable tagUnderCursor -font $font1 -fg Black -bg $bg1 -padx 20


# Arrange everything:


pack .bot -side bottom -fill x 
pack .bot.label -side left
pack .bot.pad -side right
pack .bot.dismiss -side right
pack .mid -side bottom -fill x
pack .mid.pad -side right
pack .mid.xscroll -side bottom -fill x
pack $textArea -side top -expand true -fill both
pack $textArea.yscroll -side right -fill y
pack $textArea.text -side left -fill both -expand true

set current_directory ""
set input_file_name ""
set fileChanged 0
set sgmlFormat 0
set num_sents 0

# Convert a text widget index (row.col) into an positive integer value
# Assume a max number of chars per line of 1024
proc IndexToInt {textIndex} {
    if { $textIndex != "" } {
      if {[regexp {^([0-9]+)\.([0-9]+)$} $textIndex match rowIndex colIndex]} {
	return [expr $rowIndex*1024 + $colIndex]
      } else {
	### puts "IndexToInt: Bad argument: $textIndex"
	return -1
      }
    } else {
      ###puts "IndexToInt: No argument."
      return -1
    }
}


# Return >0 if tag1 should come before tag2
#        <0 if tag2 should come before tag1
#        0  if tag1 and tag2 have the same end-points
# Assuming tags do not cross brackets
proc PreOrder { tag1 tag2 } {
  global rangeTable
  set from1 [IndexToInt [lindex $rangeTable($tag1) 0]]
  set to1   [IndexToInt [lindex $rangeTable($tag1) 1]]
  set from2 [IndexToInt [lindex $rangeTable($tag2) 0]]
  set to2   [IndexToInt [lindex $rangeTable($tag2) 1]]
  if {$from1 == $from2} {
    return [expr $to1 - $to2]
  }
  return [expr $from2 - $from1]
}


# Insert tag in a priorityList respecting the preorder sequence:
# Returns the updated list:
# Average worst case running time: O(List length = Average number of tag end-points per text index)
proc InsertInPriorityList {tag pList} {
  set size [llength $pList]
  set i 0
  set iPoint 0
  while {$i < $size} {
    if {[PreOrder [lindex $pList $i] $tag] <= 0} {
      set i $size
    } else {
      incr i
      set iPoint $i
    }
  }
  return [linsert $pList $iPoint $tag]
}


   
proc UniqueStrings { alist } {
  set result " "
  set spaces " "
  foreach elem $alist {
    if {[regexp " $elem " $result match value] == 1} {
      set result $result
    } else {
      set result $result$elem$spaces
    }
  }
  return $result
}

set tabs    "    "
set tabSize [string length $tabs]

# Load an sgml file and fills in sents, num_sents, listOfSgmlTags and rangeTable:
wm minsize . 80 30 
set numFiles [llength $inputFiles] 
set checknumber 0
$textArea.text configure -state normal
set mismatch 0
set empty "                                                                               "

for { set i 0 } { $i < $numFiles } { incr i } {
  
  ####puts "Loading file: [lindex $inputFiles $i]"
  set callFromShowSegmentation 1
  set colorTable(TopLevel) white
  set insertIndex -1

  set inputFile [lindex $inputFiles $i]
  LoadFile $inputFile
  if {$i==0} {

    set index 1
    set size 1
    set nextSize 0
    set checknumber $num_sents
    while {$index < $num_sents} {
      set spaces($index) " "
      $textArea.text insert end [format " \n"]
      incr index
    }

  } else {

    if { $checknumber != 0 } {
      
      if { $num_sents != $checknumber } {
        set tagUnderCursor "Warning: file size may be different for $inputFile ($num_sents vs. $checknumber)"
        ### puts "Warning: file size may be different for $inputFile ($num_sents vs. $checknumber)"
        set mismatch 1
      }
      
      if { $num_sents < $checknumber } {
   
	set s $num_sents
	while { $s < $checknumber } {
	  set sents($s) ""
	  incr s
	}
	
	set num_sents $checknumber
	
      } 
      
    } 
    
    set size [expr $nextSize - 1]
  }

  if {$i == [expr $numFiles - 1]} {
    set isLastFile 1
  } else {
    set isLastFile 0;
  }

  foreach oneTag $listOfSgmlTags {
    set tagHead $oneTag
    regexp {^([-a-zA-Z0-9]+)\_([0-9]+)$} $oneTag match tagHead value
    if {[string length $segmentType] > 1} {
      if {$tagHead == $segmentType} {
      lappend segmentTags $oneTag
      }
    }
    if {[lsearch -exact $segmentTags $oneTag] != -1} {
      set fromIndex [lindex $rangeTable($oneTag) 0]
      set toIndex   [lindex $rangeTable($oneTag) 1]
      regexp {^([0-9]+)\.([0-9]+)} $fromIndex match fromRow fromCol
      regexp {^([0-9]+)\.([0-9]+)} $toIndex match toRow toCol
      lappend listOfSegmentTags $oneTag
      # Assign color:
      if {[info exists colorCode($fromRow.$toRow)]} {
	set colorTable($oneTag) [lindex $palette $colorCode($fromRow.$toRow)]
	### : $oneTag $fromRow.$toRow [lindex $palette $colorCode($fromRow.$toRow)]"
      } else {
	if {![info exists colorTable($oneTag)]} {
	  set colorTable($oneTag) [lindex $other_palette $colorIndex]
	  ### puts "Not found: $oneTag $fromRow.$toRow [lindex $palette $colorIndex]"
	  incr colorIndex
	  set  colorIndex [expr $colorIndex % $other_size]
	}
      }
      # Insert tag end-points in priority lists:
      set plist [list]
      if {[info exists beginAtRow($fromRow)]} {
	set plist $beginAtRow($fromRow)
      }
      set beginAtRow($fromRow) [InsertInPriorityList $oneTag $plist]
      set plist [list]
      if {[info exists endAtRow($toRow)]} {
	set plist $endAtRow($toRow)
      }
      set endAtRow($toRow) [InsertInPriorityList $oneTag $plist]
    }
  }

  set segments(0) [list TopLevel]
  set colorTable(TopLevel) white
  set maxDepth 0
  set oldRow 0


  for {set row 1} {$row < $num_sents} {incr row ; incr oldRow} {
    # Copy over all segments from row-1 that are still open:
    foreach oldSegment $segments($oldRow) {
      if {[info exists endAtRow($oldRow)]} {
   if {[lsearch -exact $endAtRow($oldRow) $oldSegment] != -1} {
     # Do nothing
   } else {
     lappend segments($row) $oldSegment
   }
      } else {
   lappend segments($row) $oldSegment
      } 
    }
    # Add the new segments that are open at new row:
    if {[info exists beginAtRow($row)]} {
      set segmentBuffer  $segments($row) 
      set segments($row) [concat $segments($row) $beginAtRow($row)]
    }
    # Update maxDepth:
    if {[llength $segments($row)] > $maxDepth} {
      set maxDepth [llength $segments($row)]
    }
  }

  ### puts "maxDepth is: $maxDepth"

  for {set row 1} {$row < $num_sents} {incr row} {
    set realSegments($row) $segments($row)
    # Normalize segments($row) to maxDepth:
    while {[llength $segments($row)] < $maxDepth} {
      lappend segments($row) [lindex $segments($row) [expr [llength $segments($row)] - 1]]
    }
  }
  

  for {set row 1} {$row < $num_sents} {incr row} {
    set col 0
    set spaces($row) ""
    # Indentation:
    foreach segment $segments($row) {
      set spaces($row) $spaces($row)$tabs
    }
  }
  

  # Display text:
  
  set index 1
  while {$index < $num_sents} {
    if {$isLastFile == 1} {
      $textArea.text insert "$index.$size" [format "%s %s " $spaces($index) $sents($index)]
    } else {
      $textArea.text insert "$index.$size" [format "%s " $spaces($index)]
      set thisSize [string length [$textArea.text get $index.0 "$index.0 lineend"]]
      if {$nextSize < $thisSize} {
         set nextSize $thisSize
      }
    }
    incr index
  }

  # Add colors:

  for {set row 1} {$row < $num_sents} {incr row} {
    set col $size
    # Add colors for Indentation:
    foreach segment $segments($row) {
      set fromIndex "$row.$col"
      set col [expr [expr $col + $tabSize] - 1]
      set toIndex [$textArea.text index "$row.$col + 1 chars"]
      incr col
      $textArea.text tag add $i.$segment $fromIndex $toIndex
      $textArea.text tag configure $i.$segment -foreground $text_bg -background $colorTable($segment)
      set idTable($i.$segment) [file tail $inputFile]
      $textArea.text tag bind $i.$segment <Enter> {
         ListTagsUnderCursor
         break
      }
    }
    set x [expr $row % 5]
    if {$x == 0} {
      $textArea.text see $row.0
      update idletasks
    }
  }

  # clean-up
  unset segments
  unset sents
  unset num_sents
  catch {unset colorTable}
  catch {unset rangeTable}
  catch {unset realSegments}
  catch {unset spaces}
  catch {unset listOfSgmlTags}
  catch {unset beginAtRow}
  catch {unset endAtRow}
  catch {unset cacheFrom}
  catch {unset cacheTo}

}

if {$mismatch == 1} {
  set tagUnderCursor "Warning: the input files are not aligned!"
} else {
  if { $numFiles > 1 } {
    if {$alignedFiles != 0} {
      set tagUnderCursor "Aligned segments across different files share the same color"
    } else {
      set tagUnderCursor "The color scheme for each file is independent"
    }
  } else {
    set tagUnderCursor "Imagination is more important than knowledge"
  }
}


$textArea.text configure -state disabled
