#####################################################################################################
# FILE:    display_segments.tcl
# PURPOSE: Add or delete a segment (and text indentation) and update all the other tags in
#          beginAtIndex() and endAtIndex() and cacheFrom() and cacheTo() tables
# AUTHOR:  Giovanni Flammia 
#
#         (N)[b] Copyright (c) 1995-1996
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
####################################################################################################

puts "Loading: display_segments.tcl..."


set tabString "    "
set strlen [string length $tabString]

# Add segment tag "name" from text index "from" to text index "to"
# Adding tabString for indentation.
# Assuming line-by-line tags. Update the segments() array and also all
# the beginAtIndex(), endAtIndex(), cacheFrom() and cacheTo() arrays

proc AddSegment {name from to when} {
  
  global strlen tabString sents segments speaker textArea colorTable
  global choiceTable tabString beginAtIndex endAtIndex idTable
  global cacheFrom cacheTo rangeTable refTable refTags refCounter isRefTag
  
  if {[info exists choiceTable(SegmentTags)]} {
    
    
    regexp {^([-a-zA-Z0-9_]+)_([0-9]+)$} $name match head value
    if {[lsearch -exact $choiceTable(SegmentTags) $head] != -1} {
      regexp {^([0-9]+)\.([0-9]+)} $from match fromRow fromCol
      regexp {^([0-9]+)\.([0-9]+)} $to match toRow toCol
      while {[expr [expr $fromRow < $toRow] && [expr $toCol <= 1]] == 1} {
         set to  [$textArea.text index "$to - 1 chars"]
         regexp {^([0-9]+)\.([0-9]+)} $to match toRow toCol
      } 
      $textArea.text configure -state normal
      for {set row $fromRow} {$row <= $toRow} {incr row} {
        # hack, works only for Segment/SubSegment
        if {[regexp {SubSegment} $name match]} {
          lappend segments($row) $name
        } else {
          set segments($row) [linsert $segments($row) 0 $name]
        }
        if {[string compare $when "online"] == 0} {
          $textArea.text insert $row.0 $tabString
        }
        # display segments
        set firstIndex $row.0
        set lastSegment ""
        foreach s $segments($row) {
          set lastIndex [$textArea.text index "$firstIndex + $strlen chars"]
          $textArea.text tag add hidden.$s $firstIndex  $lastIndex 
          $textArea.text tag configure hidden.$s -foreground Black -background $colorTable($s) 
          set firstIndex $lastIndex
          set lastSegment $s
        } 
        if {[string compare $lastSegment ""] != 0} {
          $textArea.text tag add hidden.color.$lastSegment $firstIndex [$textArea.text index "$row.0 lineend"] 
          $textArea.text tag configure hidden.color.$lastSegment -foreground $colorTable($s) -background DarkSlateBlue 
          $textArea.text tag raise hidden.color.$lastSegment
          $textArea.text tag raise hidden.$lastSegment
        }
      }
      $textArea.text configure -state disabled
      if {[string compare $when "online"] == 0} {
        for {set row $fromRow} {$row <= $toRow} {incr row} {
          set last [$textArea.text index "$row.0 lineend - 1 chars"]
          regexp {^([0-9]+)\.([0-9]+)$} $last match thisrow lastcol
          for {set col 0} {$col <= $lastcol} {incr col} {
            if {[info exists beginAtIndex($row.$col)]} {
              set newcol [expr $col + $strlen]
              set newBegin($row.$newcol) $beginAtIndex($row.$col)
              unset beginAtIndex($row.$col)
            }
            if {[info exists endAtIndex($row.$col)]} {
              set newcol [expr $col + $strlen]
              set newEnd($row.$newcol) $endAtIndex($row.$col)
              unset endAtIndex($row.$col)
            }
          }
        }
        for {set row $fromRow} {$row <= $toRow} {incr row} {
          set last [$textArea.text index "$row.0 lineend"]
          regexp {^([0-9]+)\.([0-9]+)$} $last match thisrow lastcol
          set lastcol [expr $lastcol + $strlen]
          for {set col 0} {$col <= $lastcol} {incr col} {
            if {[info exists newBegin($row.$col)]} {
              set x [IndexToInt "$row.$col"]
              set beginAtIndex($row.$col) $newBegin($row.$col)
              foreach item $beginAtIndex($row.$col) {
                set cacheFrom($item) $x
                set oldStart [lindex $rangeTable($item) 0]
                set oldEnd   [lindex $rangeTable($item) 1] 
                set rangeTable($item) [list "$row.$col" "$oldEnd"]
                if {[info exists isRefTag($item)]} {
                  set oldKey "$oldStart $oldEnd"
                  set newKey "$row.$col $oldEnd"
                  catch {unset refTable($key)}
                  set refTable($newKey) $idTable($item)
                  #### puts "moved $idTable($item) to $newKey"
                }
              }
            } 
            if {[info exists newEnd($row.$col)]} {
              set x [IndexToInt "$row.$col"]
              set endAtIndex($row.$col) $newEnd($row.$col)
              foreach item $endAtIndex($row.$col) {
                set cacheTo($item) $x
                set oldStart [lindex $rangeTable($item) 0]
                set oldEnd   [lindex $rangeTable($item) 1] 
                set rangeTable($item) [list "$oldStart" "$row.$col"]
                if {[info exists isRefTag($item)]} {
                  set oldKey "$oldStart $oldEnd"
                  set newKey "$oldStart $row.$col"
                  catch {unset refTable($oldKey)}
                  set refTable($newKey) $idTable($item)
		  ### puts "moved $idTable($item) to $newKey"
                } 
              }
            }
          }
        }
      }
    }
  }
}

# Delete segment tag "name" from text index "from" to text index "to"
# Removing tabString for indentation
# Assuming line-by-line tags. Update the segments() array and also all
# the beginAtIndex() and endAtIndex() and cacheFrom() and caheTo() arrays

proc DeleteSegment {name from to} {

  global strlen tabString sents segments idTable hiddenTags 
  global speaker textArea choiceTable beginAtIndex endAtIndex
  global cacheFrom cacheTo colorTable rangeTable refTable refTags refCounter isRefTag
  
  set subSegmentWarning ""
  if {[info exists choiceTable(SegmentTags)]} {
    regexp {^([-a-zA-Z0-9_]+)_([0-9]+)$} $name match head value
    if {[lsearch -exact $choiceTable(SegmentTags) $head] != -1} {
      regexp {^([0-9]+)\.([0-9]+)} $from match fromRow fromCol
      regexp {^([0-9]+)\.([0-9]+)} $to match toRow toCol
      while {[expr [expr $fromRow < $toRow] && [expr $toCol <= 1]] == 1} {
        set to  [$textArea.text index "$to - 1 chars"]
        regexp {^([0-9]+)\.([0-9]+)} $to match toRow toCol
      }
      $textArea.text configure -state normal
      $textArea.text tag delete hidden.$name
      $textArea.text tag delete hidden.color.$name
      for {set row $fromRow} {$row <= $toRow} {incr row} {
        # clear display:
        set index [lsearch -exact $segments($row) $name]
        if {$index != -1} {
          while {$index != -1} {
            set segments($row) [lreplace $segments($row) $index $index]
            set index [lsearch -exact $segments($row) $name]
          }
          if {[regexp {SubSegment} $segments($row) match]} {
            set subSegmentWarning "$idTable($segments($row))"
          }
          set last [$textArea.text index "$row.0 lineend"]
          regexp {^([0-9]+)\.([0-9]+)$} $last match thisrow lastcol
          set lastcol [expr $lastcol + $strlen]
          for {set col $strlen} {$col <= $lastcol} {incr col} {
            if {[info exists beginAtIndex($row.$col)]} {
              set newBegin($row.[expr $col - $strlen]) \
                  $beginAtIndex($row.$col)
              unset beginAtIndex($row.$col)
            }
            if {[info exists endAtIndex($row.$col)]} {
              set newEnd($row.[expr $col - $strlen]) \
                  $endAtIndex($row.$col)
               unset endAtIndex($row.$col)
            }
           }
          for {set col 0} {$col <= $lastcol} {incr col} {
            if {[info exists newBegin($row.$col)]} {
                  set x [IndexToInt "$row.$col"]
              set beginAtIndex($row.$col) $newBegin($row.$col)
                  foreach item $beginAtIndex($row.$col) {
                    set cacheFrom($item) $x
                    set oldStart [lindex $rangeTable($item) 0]
                    set oldEnd   [lindex $rangeTable($item) 1] 
                    set rangeTable($item) [list "$row.$col" "$oldEnd"]
                    if {[info exists isRefTag($item)]} {
                      set oldKey "$oldStart $oldEnd"
                      set newKey "$row.$col $oldEnd"
                       catch {unset refTable($key)}
                      set refTable($newKey) $idTable($item)
                      ### puts "moved $idTable($item) to $newKey"
                    }
                  }
            } 
            if {[info exists newEnd($row.$col)]} {
              set x [IndexToInt "$row.$col"]
              set endAtIndex($row.$col) $newEnd($row.$col)
              foreach item $endAtIndex($row.$col) {
                set cacheTo($item) $x
                set oldStart [lindex $rangeTable($item) 0]
                set oldEnd   [lindex $rangeTable($item) 1] 
                set rangeTable($item) [list "$oldStart" "$row.$col"] 
                if {[info exists isRefTag($item)]} {
                  set oldKey "$oldStart $oldEnd"
                  set newKey "$oldStart $row.$col"
                  catch {unset refTable($oldKey)}
                  set refTable($newKey) $idTable($item)
                  #### puts "moved $idTable($item) to $newKey"
                } 
              }
            }
          }
          # hack works only for Segment/SubSegment
          $textArea.text delete $row.0 $row.$strlen
        }
      }
      $textArea.text configure -state disabled
    }
  }
  
  if {[regexp {SubSegment} $subSegmentWarning match]} {
    WarningMessage "You are changing the top-level segment structure. \n Make sure that $subSegmentWarning is contained into the appropriate Segment."
    bell 
  }
  
}









