#####################################################################################################
# FILE:    edit_menu_handlers.tcl
# PURPOSE: handlers for the Edit pull-down menu
# AUTHOR:  Giovanni Flammia 
#
#         (N)[b] Copyright (c) 1995
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
####################################################################################################

puts "Loading: edit_menu_handlers.tcl..."

proc Untag {} {

    global oldTag thisTag cacheRange cacheTag clipBoard fileChanged
    global textArea tagUnderCursor selectedId cacheTag entryTag idTable 
    global refTags refTable refCounter tagUnderCursor statusMessage

    puts "ClipBoard: $clipBoard"
    if {[llength $clipBoard] > 0} {
      set oldTag [lindex $clipBoard 0]
      set thisTag $oldTag
      set selectedId $idTable($thisTag)
      set cacheTag($selectedId) $thisTag
      set cacheRange($thisTag) [$textArea.text tag ranges $thisTag]
      set fromIndex [lindex $cacheRange($thisTag) 0]
      $textArea.text see $fromIndex
      set tagUnderCursor "Deleting Tag: $idTable($thisTag)..."
      update idletasks
      set fileChanged 1
      DeleteThisTag
    } else {
      set tagUnderCursor "Can't untag: no edited tags to delete."
      update idletasks
    }
}


	
proc DeleteThisTag {} {
  
  global currentTags currentIndex selectedId cacheTag oldTag entryTag
  global choicesArea fileChanged fileExists textArea
  global fieldTable beginAtIndex endAtIndex cacheRange clipBoard
  global cacheFrom cacheTo touchedIndex idTable
  global refTags refTable refCounter tagUnderCursor statusMessage


  puts "Called: DeleteThisTag"

  if {$selectedId != ""} {
    if {[info exists cacheTag($selectedId)]} {
      set thisTag $cacheTag($selectedId)
      regexp {^([-a-zA-Z0-9_]+)_([0-9]+)$} $thisTag match tagName tagValue 
      set fileChanged 1
      set fileExists 1
      set fromIndex   [lindex $cacheRange($thisTag) 0]
      set toIndex     [lindex $cacheRange($thisTag) 1]
      set currentTags [DeleteFromPriorityList $thisTag $currentTags]
      $textArea.text tag remove $thisTag $fromIndex $toIndex
      set beginAtIndex($fromIndex) [DeleteFromPriorityList $thisTag $beginAtIndex($fromIndex)]
      set endAtIndex($toIndex)     [DeleteFromPriorityList $thisTag $endAtIndex($toIndex)]
      set oldValue [lindex $fieldTable($tagName) 0]
      set tagFound [lsearch -exact $clipBoard $thisTag]
      if {$tagFound >= 0} {
	set clipBoard [lreplace $clipBoard $tagFound $tagFound]
      }
      if { $oldValue == $tagValue } {
	set fieldTable($tagName) [list [expr $oldValue - 1]]
      }
      DeleteSegment $thisTag $fromIndex $toIndex
      set refExists [CheckForRefTags "Ref$tagName" $idTable($thisTag) $fromIndex $toIndex]
      if { $refExists == 1} {
	set refCounter [expr $refCounter + 1]
	set statusMessage "$refCounter hidden tags left."
      }
      set selectedId  ""
      set oldTag      ""
      set entryTag    ""
      if {[$textArea.text tag ranges ViewTags.sel] != ""} {
	$textArea.text tag delete ViewTags.sel
      }
      unset cacheFrom($thisTag)
      unset cacheTo($thisTag)
      FindAllTags
      $choicesArea.directory see $touchedIndex
      set editingAction "Edit Tags"
      .chwindow config -cursor top_left_arrow
      update idletasks
    }
  }
}


proc NameThisTag {} {

  global selectedId cacheTag oldTag selectedId entryTag
  global choicesArea fileChanged fileExists textArea editingAction
  global fieldTable beginAtIndex endAtIndex cacheRange touchedIndex 
  global refTags refTable refCounter tagUnderCursor statusMessage

  puts "called NameThisTag"

  if {$selectedId != ""} {
    if {[info exists cacheTag($selectedId)]} {
      set thisTag $cacheTag($selectedId)
      .chwindow config -cursor watch
      set editingAction "Name Tag"
      EditMenuHandler
      set editingAction "Edit Tags"
      FindAllTags
      $choicesArea.directory see $touchedIndex
      .chwindow config -cursor top_left_arrow
      wm withdraw .chwindow
      update idletasks
    }
  }
}


proc AddThisTag {} {
  
  global selectedId cacheTag oldTag selectedId entryTag
  global choicesArea fileChanged fileExists textArea editingAction
  global fieldTable beginAtIndex endAtIndex cacheRange
  
  if {$entryTag != ""} {
    if {$oldTag != ""} {
      set editingAction "Add New Tag"
      EditMenuHandler
      set editingAction "Add New Tag"
    }
  }
}



proc EditMenuHandler {} {

  global editingAction entryTag oldTag fileChanged 
  global idTable cacheTag textArea cacheRange 
  global menuWidgets refTags refTable 
  global refCounter tagUnderCursor statusMessage

  puts "Called EditMenuHandler"

  switch $editingAction {
    "Name Tag"
    {
      if {$entryTag != ""} {
	if {$oldTag != ""} {
	  if {[info exists cacheTag($oldTag)]} {
	    set fileChanged 1
	    set thisTag $cacheTag($oldTag)
	    set idTable($thisTag) "$thisTag $entryTag"
	    set fromIndex [lindex $cacheRange($thisTag) 0]
	    set toIndex   [lindex $cacheRange($thisTag) 1]
	    set colorList [SelectColor $thisTag $idTable($thisTag) $fromIndex $toIndex] 
	    $textArea.text tag configure $thisTag -foreground [lindex $colorList 0] 
	    set entryTag ""
	    set oldTag   ""
	  }
	}
      }
    }
    "Add New Tag"
    {
      if {$entryTag != ""} {
	if {$oldTag != ""} {
	  regsub -all {^[ ]+} $entryTag ""  entryTag
	  regsub -all {[ ]+$} $entryTag ""  entryTag
	  regexp {([-a-zA-Z0-9]+)$} $oldTag match item
	  if {[info exists menuWidgets($oldTag)]} {
	    AddCommandToMenus $item $menuWidgets($oldTag) $entryTag
	  } else {
	    WarningMessage "Can't find Menus for: \"$oldTag\""
	  }
	}
      }
    }
  }
}



