###########################################################################################
# FILE:    file_functions.tcl
# PURPOSE: Load a text or annotated file, Save annotated file and Quit program
# AUTHOR:  Giovanni Flammia
#
#         (N)[b] Copyright (c) 1995
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
###########################################################################################


### puts "Loading: file_functions.tcl..."

# changed this to work also under Windows
proc FindFileOrPattern {lbox pattern} {
   global current_directory
   
   if {[info exists current_directory]} {
        set directory $current_directory
   } else {
        set current_directory [my_pwd]
        set directory $current_directory
   }
   
   set newline "\n"
   set slash   "/"
   set allfiles "" 
   
   if {[string compare $pattern ""] == 0} {
      set pattern *
   }
   
   # adapted to work on all platforms 
   # regsub -all "/" $directory "\\" d 
   
   my_cd $current_directory
   if [catch {set allfiles [lsort [glob -nocomplain -- *$pattern*]]} ] {
      set allfiles ""
   }
   
   if {$allfiles != ""} {
  
     $lbox delete 0 end
     $lbox insert end ".."
     foreach filename $allfiles {
        if {[string compare $filename ""] != 0} {
           if {[file isdirectory $current_directory/$filename]} {
              set filename $filename$slash
           }
           $lbox insert end $filename
        } 
     }
  
   } else {
      WarningMessage "In $directory no files or folders matched the desired pattern: $pattern"
   }
  
}


# fill listbox_window with directory listing:
proc DisplayFiles {listbox_window current_directory} {
    
    set listfiles ""
    set newline "\n"
    set slash   "/"
    $listbox_window delete 0 end
 
    my_cd $current_directory
    if [catch {set listfiles [lsort [glob -nocomplain *]]} ] {
      set allfiles ""
    }
    
    $listbox_window insert end ".."
    foreach filename $listfiles {
        if {[string compare $filename ""] != 0} {
            if {[file isdirectory $current_directory/$filename]} {
                set filename $filename$slash
            }
            $listbox_window insert end $filename
        } 
    }
}



# Load a secondary generic text file (for .second pop-up window 
# or for .third.middle.msg for display of source file or help text)
# and puts it into textBuffer() array of size sizeBuffer
proc LoadTextFile {file_name} {
    global sizeBuffer textBuffer
    if {[regexp {([a-zA-Z]+)} $file_name match value]} {
    regsub {[@\*]$} $file_name "" clean_file_name
    if [catch {file exists $clean_file_name}] {
        WarningMessage "Can't read file: $clean_file_name. Check permissions or path name."
        return 0
    }
    if {[file exists $clean_file_name]} {
        puts "Called LoadTextFile $file_name"
        set file [open $file_name r]
        set sizeBuffer 1
        while {[gets $file line] >= 0} {
        set textBuffer($sizeBuffer) $line
        # puts "$textBuffer($sizeBuffer)"
        incr sizeBuffer
        }
        close $file
        return 1
    } else {
        puts "LoadTextFile: $file_name not found."
        return 0
    }
    }
}
 


proc ProcessAliases {text} {
    global choiceTable formatTable aliasNumber
    if {[info exists choiceTable(Alias)]} {
    set unders "_"
    set space  \\s
    switch -regexp $text {
        {[-a-zA-Z;.,:_]+} {
        foreach pattern $choiceTable(Alias) {
            regsub $space $pattern " " inputPattern 
            if {[regexp "$inputPattern" $text]} {
            if {[info exists formatTable($pattern)]} {
                # puts "Pattern: $inputPattern Subs: $formatTable($pattern)"
                if {[regexp {^<([-a-zA-Z0-9]+)=\"([^\"]+)\">([^<]+)</([-a-zA-Z0-9]+)>} \
                     $formatTable($pattern) match tag1 id new tag2]} {
                if {[info exists aliasNumber($tag1)]} {
                    incr aliasNumber($tag1) 
                } else {
                    set aliasNumber($tag1) 1
                }
                set n $aliasNumber($tag1)
                set subs "<$tag1$unders$n=\"$id\">$new</$tag1$unders$n>"
                } else {
                if {[regexp {^<([-a-zA-Z0-9]+)>([^<]+)</([-a-zA-Z0-9]+)>} \
                     $formatTable($pattern) match tag1 new tag2]} {
                    if {[info exists aliasNumber($tag1)]} {
                    incr aliasNumber($tag1) 
                    } else {
                    set aliasNumber($tag1) 1
                    }
                    set n $aliasNumber($tag1)
                    set subs "<$tag1$unders$n>$new</$tag1$unders$n>"
                } else {
                    set subs $formatTable($pattern)
                }
                }
            } else {
                set subs ""
            }
            # puts   "Alias: \"$pattern\" -> \"$subs\""
            regsub "$inputPattern" $text $subs text
            }
        }
        }
    }
    }
    return $text
}


# Load the main text or SGML file and puts it into sents(), speaker() and segments()
# arrays, creating various Tag hash tables
# and filling in rangeTable(X) = [list fromIndex(X) toIndex(X)]
# The beginAtIndex() and endAtIndex() information is completed by the function
# AddTagsFromTables in edit_functions.tcl
# Recognized SGML/Nb format (but is it standard sgml? no)
# ---------------------------------------------------------
# <! Header Lines:
#    (N)[b] Creation Date
#    Format: <Tag_Number>Text</Tag_Number> or <Tag_Number="Identifier">Text</Tag_Number>
#    Config: source config file
#    Source: input file
#    Author: user name
# !>
# <SGML>
# Tagged Text Lines
# </SGML>
# ---------------------------------------------------------
# Legal Tag: [A-Za-z0-9]+_[0-9]+ examples: Segment_1 NP_2 VP_32
# Legal Id:  [^<>"]+
# Tagged text: <Tag_#>...</Tag_#> or: <Tag_#="Id">...</Tag_#>
# Any other SGML tags are ignored!
#
set TimeStamp ""
proc LoadFile {file_name} {

    global current_index 
    global clipBoard tagUnderCursor fileValue sourceName cacheFrom cacheTo sourceDir choiceTable 
    global formatTable aliasNumber tagsWithoutNumbers
    global num_sents output_file_name fieldTable cacheFrom cacheTo currentTags currentIndex
    global sents speaker segments textArea current_directory input_file_name fileChanged 
    global idTable rangeTable sgmlFormat listOfSgmlTags beginAtIndex endAtIndex
    global TimeStamp

    puts "Called LoadFile dir: $current_directory file: $file_name\n"

    set sgmlFormat 0
    set sourceName ""
    set sourceDir ""
    regsub {[@\*]$} $file_name "" clean_file_name
    if {[regexp {\/} $clean_file_name]} {
      set input_file_name $clean_file_name
    } else {
      set input_file_name $current_directory$clean_file_name
    }
    if {![file readable $input_file_name]} {
      set input_file_name $current_directory/$clean_file_name
    }
    if {[file readable $input_file_name]} {
      set file [open $input_file_name r]
      set num_sents 1
      set state "isText"
      set sgmlFormat 0
      set underscore "_"
      set listOfSgmlTags [list]
      set beginAtIndex(0) 0
      set endAtIndex(0) 0
      set currentIndex(0) 0
      set cacheFrom(0) 0
      set cacheTo(0) 0
      catch {unset beginAtIndex}
      catch {unset endAtIndex}
      catch {unset currentIndex}
      catch {unset cacheFrom}
      catch {unset cacheTo}
      catch {unset tagsWithoutNumbers}
      catch {unset rangeTable}
      set tagsWithoutNumbers     [list]
      set currentTags            [list]
      set currentIndex(TopLevel) 0
      set cacheFrom(TopLevel)    0
      set cacheTo(TopLevel)      0
      set clipBoard [list]
      set current_index 1.0
      if {[info exists choiceTable(SegmentTags)]} {
        set theSegmentTags $choiceTable(SegmentTags)
      } else {
        set theSegmentTag [list]
      }
      # WarningMessage "Loading input file: $file_name"
      if {[info exists choiceTable(Alias)]} {
        foreach p $choiceTable(Alias) {
	  puts "Alias: \"$p\" -> \"$formatTable($p)\""
        }
      }
      if {[info exists segments]} {
        unset segments
      }
      foreach tagIndex [array names fieldTable] {
        set fieldTable($tagIndex) [list 0]
      }
      set segments(0) [list]
      set segmentBuffer [list]
      set fileValue ""
      set output_file_name ""
      set tagUnderCursor "Loading: [file tail $input_file_name]..."
      update idletasks
      if {[catch {set TimeStamp [clock format [clock seconds]]}]} {
	set TimeStamp ""
      }
      while {[gets $file line] >= 0} {
        # Process aliases: 
        if {[info exists choiceTable(Alias)]} {
	  set newline [ProcessAliases $line] 
	  while {$newline != $line} {
            set line $newline 
            set newline [ProcessAliases $line]
	  }
        }
        # state identifies line content:
        switch -regexp -- $line {
	  {^[ ]*\<!\-.+\-\>[ ]*$} { set state "isText" }
	  {^<!}       { set state "isHeader" }
	  {^[^!]*!>}  { set state "nil"}
	  {^<SGML>}   { set state "willBeText"}
	  {^</SGML>}  { set state "nil" }
	  {<.+>}      { if {[regexp {Text} $state]} {set state "isTaggedText" }}
	  {[-a-zA-Z;.,:_]+} { if {[regexp {Text} $state]} {set state "isText"}}
        }
	if {[string compare $line ""] == 0} {
	  if {[regexp {Text} $state]} {set state "isText"}
	}
        # puts "$state: $line"
        # Process only (tagged) text line:
        switch $state {
          "isHeader" 
	  {
            switch -regexp -- $line {
	      {Source: [-_a-zA-Z;.,:/0-9]+} {
                regexp {Source: ([-_a-zA-Z;.,:/0-9]+)} $line match sourceFile
                regexp {^([-_a-zA-Z;.,:/0-9]+)/([-_a-zA-Z\.0-9]+)$} $sourceFile match sourceDir sourceName
	      }
            }
	  }
	  "isText" 
	  {
            set segments($num_sents) [list]
            set sents($num_sents) $line
            incr num_sents
	  }
	  "isTaggedText"
	  {
            set segments($num_sents) [list]
            set outputText ""
            set openTag  ""
            set closeTag ""
            set restLine ""
            set idTag    ""
            set someText ""
            set match    ""
            set row $num_sents
            set col 0
            while {$line != ""} {
	      if {[regexp {^<[ ]*([-_a-zA-Z0-9]+)[ ]*=[ ]*\"([^<>\"]+)\"[ ]*>(.*)} $line match openTag idTag restLine]} {
                # <openTag="idTag">restLine idTag cannot contain < > "
                # puts "open Tag \= idTag: <$openTag=\"$idTag\"> restLine: $restLine"
                set line                 $restLine
                set idTable($openTag)    "$openTag $idTag"
                set col                  [string length $outputText]
                set rangeTable($openTag) [list $row.$col]
                set sgmlFormat 1
                if {[regexp {^([-a-zA-Z0-9]+)\_([0-9]+)$} $openTag match head number]} {
		  if {[info exists fieldTable($head)]} {
                    if {[lindex $fieldTable($head) 0] < $number} {
		      set fieldTable($head) [list $number]
                    }
		  } else {
                    set fieldTable($head) [list $number]
		  }
		  if { $head == "Speaker" } {
                    set speaker($row) $idTag
		  }
		  lappend listOfSgmlTags $openTag
		  lappend stack($head) $number
                } elseif {[regexp {^([-a-zA-Z0-9]+)$} $openTag match head]} {
		  set number 1
		  if {[info exists fieldTable($head)]} {
                    set number [expr $fieldTable($head) + 1]
		  }
		  set fieldTable($head) [list $number]
		  set openTag "$head$underscore$number"
		  if { $head == "Speaker" } {
                    set speaker($row) $idTag
		  }
                  lappend listOfSgmlTags     $openTag
		  lappend stack($head)       $number
		  lappend tagsWithoutNumbers $openTag
		  set line                 $restLine
		  set idTable($openTag)    "$openTag $idTag"
		  set col                  [string length $outputText]
		  set rangeTable($openTag) [list $row.$col]
                }
	      } elseif \
		  {[regexp {^<[ ]*([-_a-zA-Z0-9]+)\_([0-9]+)[ ]*>(.*)} $line match head number restLine]} {
		    # <openTag>restLine
		    # puts "simple Open Tag: <$openTag> restLine: $restLine"
		    set openTag "$head$underscore$number"
		    set line $restLine
		    set idTable($openTag)    $openTag
		    set col                  [string length $outputText]
		    set rangeTable($openTag) [list $row.$col]
		    set sgmlFormat 1
		    if {[regexp {^([-a-zA-Z0-9]+)\_([0-9]+)$} $openTag match head number]} {
                      if {[info exists fieldTable($head)]} {
			if {[lindex $fieldTable($head) 0] < $number} {
			  set fieldTable($head) [list $number]
			}
		      } else {
			set fieldTable($head) [list $number]
		      }
                      lappend listOfSgmlTags $openTag
		      lappend stack($head) $number
		    } elseif {[regexp {^([-a-zA-Z0-9]+)$} $openTag match head]} {
		      # puts "? simple tag ? <$head>"
		      set number 1
                      if {[info exists fieldTable($head)]} {
			set number [expr $fieldTable($head) + 1]
		      }
		      set fieldTable($head) [list $number]
		      set openTag "$head$underscore$number"
		      if { $head == "Speaker" } {
			set speaker($row) $idTag
		      }
		      lappend listOfSgmlTags     $openTag
		      lappend stack($head)       $number
		      lappend tagsWithoutNumbers $openTag
		      set line                 $restLine
		      set idTable($openTag)    "$openTag"
		      set col                  [string length $outputText]
		      set rangeTable($openTag) [list $row.$col]
		    }
		  } elseif \
		  {[regexp {^<[ ]*/([-_a-zA-Z0-9]+\_[0-9]+)[ ]*>(.*)} $line match closeTag restLine]} {
		    # </closeTag>restLine           
		    # puts "close Tag: </$closeTag> restLine: $restLine"
		    set     line  $restLine
		    set     col   [string length $outputText]
		    set tagUnderCursor "Reading: $closeTag"
		    update idletasks
		    if {[info exists rangeTable($closeTag)]} {
		      set rangeTable($closeTag) [concat $rangeTable($closeTag) [list $row.$col]] 
		    }
                  } elseif {[regexp {^([^<]+)(<.+)} $line match someText restLine]} {
		    # someText<
		    # puts "some Text: $someText rest Line: $restLine"
		    set outputText $outputText$someText
		    set line $restLine
		  } elseif {[regexp {^(<[^<]+>[^<]*)(<.+)} $line match someText restLine]} {
		    # unrecognized tag with someText<
		    # puts "unrecognized tag and some Text: $someText rest Line: $restLine"
		    set outputText $outputText$someText
		    set line $restLine
		  } else {
		    # default
		    set outputText $outputText$line
		    # puts "default: $line"
		    set line ""
		  }
            }
            if {$outputText != ""} {
              if {$state != "isHeader"} {
                set sents($num_sents) $outputText
                incr num_sents
              }
            }
	  }
        }
      }
      close $file
      set fileChanged 0
      set tagUnderCursor "Loaded: [file tail $input_file_name]"
      if {$sourceName == ""} {
        set sourceName [file tail $input_file_name]
        regexp {([-_a-zA-Z;.,:/0-9]+)$sourceName$} $input_file_name match sourceDir
      }
      return 1
    } else {
      WarningMessage "Can't read $input_file_name (file not found or no read permission)"
      return 0
    }
  }


# Load a configuration file and save config parameters into glob vars hashtables;
# recognized format for config file:
# --------------------------------------------------
# #FIELD
#      Field1
#      ...
#      FieldN
# #CHOICES
#      Choice1 [Value1]
#      ...
#      ChoiceM [ValueM]
# #HELP
#      Help message
# #END
# ---------------------------------------------------
proc LoadConfigFile {file_name fieldTable choicesTable helpTable} {
    upvar \#0 $fieldTable   fields
    upvar \#0 $choicesTable choices
    upvar \#0 $helpTable    helps
    global formatTable
    puts "Loading Configuration File: [file tail $file_name]"
    if {[file exists $file_name]} {
    set file [open $file_name r]
    set state     "nil"
    set thisField "nil"
    while {[gets $file line] >= 0} {
        switch -regexp $line {
        {^\#FIELD}    {set state "readField" ; set fieldList {} }
        {^\#CHOICES}  {set state "readChoices"}
        {^\#TABLE}    {set state "readChoices"}
        {^\#HELP}     {set state "readHelp"}
        {^\#END}      {set state "nil"}
        {[a-zA-Z0-9]+}
        {  
          regsub {^( |\t)+} $line "" cleanLine
          set value ""
          set secondValue ""
          set count [regexp {([-a-zA-Z0-9{}$\"\/\\<>_=:?.;,!^]+)} $cleanLine match value]
          set count [expr $count + [regexp {([^ ]+)[^ ]+[ ]+([^ ].+)} $cleanLine match firstValue secondValue]]
          switch $state {
            "readField" { 
              set fields($value)  [list 0]
              set choices($value) {}
              set helps($value)   {}
              lappend fieldList $value
            }
            "readChoices" { 
              if {$count >= 1} {
                foreach thisField $fieldList {
                  lappend  choices($thisField) $value 
                }
                if {[regexp {[a-zA-Z0-9]} $secondValue]} {
                  if {[info exists formats($value)]} {
                    puts "**** Parsing Error: two values for ($value) 1($formats($value)) and 2($secondValue)"
                    puts "**** Line: $line"
                  } else {
                    regsub {[ ]+$} $secondValue "" trimmedValue
                    set formatTable($value) $trimmedValue
                    }
                }
              }
            }
            "readHelp"    { 
              foreach thisField $fieldList {
                lappend  helps($thisField) $line 
              }
            }
          }
        }
          default {set state "nil"}
        }
    }
      close $file
    } else {
      puts "LoadConfigFile: $file_name not found"
    }
}


set tagsWithoutNumbers [list]

# Save a text file with tagged SGML format: 
# Format: <Tag_Number>Text</Tag_Number> or <Tag_Number=\"Identifier\">Text</Tag_Number>"
proc SaveFile {} {

    global tagUnderCursor .d TimeStamp
    global fileValue current_directory dboxEntry 
    global fileExists fileChanged output_file_name input_file_name 
    global beginAtIndex endAtIndex idTable textArea annotatorName configName tagsWithoutNumbers
    global env

    if [catch {set user_name $env(USER)}] {
      set user_name "unknown"
    }
    
    if [catch {set machine_name $env(HOST)}] {
      set machine_name "unknown"
    }

    if [catch {set dir_name $env(PWD)}] {
      set dir_name "unknown"
    }

    if $fileExists {
    if $fileChanged {
        if {($fileValue != "") || ($input_file_name != "")} {
	  set suffix ".nb"
	  set old    ".old"
	  set new    ".new"
	  set dot    "."
	  set fileExists 1
	  set fileChanged 1
	  update idletasks
	  if {[string compare $fileValue ""] != 0} {
            if {[string index $current_directory [expr [string length $current_directory] - 1]] == "/" } {
	      set output_file_name "$current_directory$fileValue"
            } else {
	      set output_file_name "$current_directory/$fileValue"
            }
	  } else {
	    puts "How come fileValue is nothing while I'm trying to save a file?"
	  }
	  puts "Saving annotated file: $output_file_name..."
	  # if output_file exists, cp it to output_file_name~
	  if {[file isdirectory $output_file_name]} {
            WarningMessage "File Save: $output_file_name is a directory! Can't save."
            return
	  }
	  if {[file isfile $output_file_name]} {
            puts "Overwriting existing file $output_file_name"
	  }
	  # Print text into output file by indices, prepending <Tag="identifier"> and
	  # appending </Tag>
	  if [catch {set file [open $output_file_name w]}] {
            WarningMessage "Can't write to: $output_file_name. Check permissions or path name."
            return
	  }
	  if {[catch {set d [clock seconds]}]} {
	    if {[catch {set d [exec date]}]} {
              set d "unknown"
            }
	  }
	  puts $file "<!\n  Nb User: $user_name Host: $machine_name Dir: $dir_name File: $output_file_name Date: $d \n  Config: $configName"
	  puts $file "  Format: <Tag_Number>Text</Tag_Number> or <Tag_Number=\"Identifier\">Text</Tag_Number>"
	  puts $file "  Source: $input_file_name\n  Author: $annotatorName\n!>\n<SGML>"
	  set currentIndex 1.0
	  set dboxEntry ""
	  set tagUnderCursor "Saving: [file tail $output_file_name]..."
	  wm title . "Nb [file tail $output_file_name]"
	  .d configure -cursor watch
	  update idletasks
	  while { $currentIndex != [$textArea.text index end] } {
            # Prefix <Tag="identifier(Tag)"> ?
            if {[info exists beginAtIndex($currentIndex)]} {
	      if {[llength $beginAtIndex($currentIndex)] > 0} {
                foreach oneTag $beginAtIndex($currentIndex) {
		  set writeTag $oneTag
		  if {[lsearch -exact $tagsWithoutNumbers $oneTag] != -1} {
                    regexp {^([-a-zA-Z0-9]+)} $oneTag match writeTag
		  }
		  if {$idTable($oneTag) == $oneTag} {
                    puts -nonewline $file "<$writeTag>"
		  } else {
                    if {[info exists idTable($oneTag)]} {
		      if {[regexp {^([-a-zA-Z0-9]+)\_([0-9]+)[ ]+([^ ].*)$} $idTable($oneTag) match head number identifier]} {
                        
                        puts -nonewline $file "<$writeTag=\"$identifier\">"
		      } else {
                        puts -nonewline $file "<$writeTag>"
		      }
                    } else {
		      puts -nonewline $file "<$writeTag>"
                    }
		  }
                }
	      }
            }
            # writing character:
            puts -nonewline $file [$textArea.text get $currentIndex]
            set currentIndex [$textArea.text index "$currentIndex + 1 chars"]
            # Suffix </Tag> ?
            if {[info exists endAtIndex($currentIndex)]} {
            if {[llength $endAtIndex($currentIndex)] > 0} {
                # Print the tags in reverse order to respect indentation:
                set t [expr [llength $endAtIndex($currentIndex)] - 1]
                while {$t >= 0} {
                set oneTag [lindex $endAtIndex($currentIndex) $t]
                set writeTag $oneTag
                if {[lsearch -exact $tagsWithoutNumbers $oneTag] != -1} {
                    regexp {^([-a-zA-Z0-9]+)} $oneTag match writeTag
                }
                puts -nonewline $file "</$writeTag>"
                set tagUnderCursor "Writing: $oneTag"
                update idletasks
                set t [expr $t - 1]
                }
	      }
            }
        }
        puts $file "</SGML>"
        close $file
        # WarningMessage "File Save: Saved output file $output_file_name"
        set fileChanged 0
        set tagUnderCursor "Saved: [file tail $output_file_name]"
        .d configure -cursor arrow
        $textArea.text configure -cursor xterm
        update idletasks
        } else {
        # WarningMessage "File Save: No input file or output file specified"
        }
    } else {
        # puts "File Save: File $output_file_name has not changed"
    }
    } else {
    # WarningMessage "File Save: No file $input_file_name to save"
    }
}




proc MayBeSaveFile {} {

    global fileLabel fileValue fileCommand fileFunction output_file_name 
    global input_file_name annotatorName suffix configPrefix current_directory
    global dboxVariable dboxEntry dboxMessage dboxOk dboxDismiss

    set dboxOk       "Save and Quit"
    set dboxDismiss  "Quit without saving"
    set dboxVariable $current_directory
    set dboxMessage  "Save changes to file?"

    set fileLabel   "Save File"
    set fileValue   ""
    set fileCommand ""
    set fileFunction "Save"
    if {[string compare $output_file_name ""] != 0} {
      set fileValue [file tail $output_file_name]
    } else {
      if {[string compare $input_file_name ""] != 0} {
	set fileValue [file tail $input_file_name]
      }
    }
    wm deiconify .dbox
    set dboxEntry $fileValue
    wm deiconify .dbox
    raise .dbox .
    update idletasks

  }


# do some global variable settings and then call LoadFile
proc ThenLoadFile {arg} {

  global fileExists fileChanged fileLabel fileValue fileCommand fileFunction
  global current_directory output_file_name input_file_name rangeTable
  global tagUnderCursor textArea sgmlFormat beginAtIndex endAtIndex
  global colorTable listOfSgmlTags statusMessage  
  
  set user_selected $arg 
  set tagUnderCursor "Loading: [file tail $user_selected]"
  puts "Try to call LoadFile with argument $user_selected"
  catch {unset rangeTable}
  set   rangeTable(TopLevel) [list]
  if {![file exists $user_selected]} {
    if {[file exists $input_file_name]} {
        set prefix [my_file_dirname $input_file_name]
        set user_selected $prefix/$user_selected
    } else {
        set prefix [my_pwd]
        set user_selected $prefix/$user_selected
    }
  } 
  puts "Loading: $user_selected"
  
  set fileExists [LoadFile $user_selected]

  regsub {[@\*]$} $user_selected "" clean_selection
  # puts "sgmlFormat: $sgmlFormat listOfTags: $listOfTags"
  if $fileExists {
    set statusMessage  "Ok"
    set tagUnderCursor "Loaded: [file tail $user_selected]"
    # ShowInputText should precede AddTagsFromTables
    $textArea.text configure -cursor watch
    update idletasks
    ShowInputText $textArea.text
    catch {wm withdraw .chwindow}
    killSegmentationWindow
    set fileChanged 0
    if { $sgmlFormat == 1} {
       set beginAtIndex(0.0)    [list]
       set endAtIndex(0.0)      [list]
       if {[info exists colorTable]} {
            unset colorTable
       }
       set colorTable(TopLevel) "Black"
       # AddTagsFromTables fills beginAtIndex and endAtIndex lists:
       ReBind
       AddTagsFromTables $listOfSgmlTags
       $textArea.text see 1.0
       $textArea.text configure -cursor xterm
       selection clear $textArea.text 
       update idletasks
    }
    
    catch {wm withdraw .d }
    catch {wm withdraw .dbox}
    
    wm title . "Nb [file tail $clean_selection]"
   
  } else {

  }
  
}


proc FirstSaveFile {} { 
    
  global fileExists fileChanged fileLabel fileValue fileCommand fileFunction
  global output_file_name input_file_name 
  
  if {$fileExists == 1} {
    if {$fileChanged == 1} {  
      set fileLabel   "Save File"
      set fileValue   ""
      set fileCommand ""
      set fileFunction "Save"
      if {[string compare $output_file_name ""] != 0} {
	set fileValue [file tail $output_file_name]
      } else {
	if {[string compare $input_file_name ""] != 0 } {
	  set fileValue [file tail $input_file_name]
	}
      }
      SaveFile
    }
  }
}

