#!/usr/local/bin/wish40

#####################################################################################################
# FILE:    pcng_draw.tcl
# PURPOSE: Draw a parsed or partially parsed sentence on a canvas widget
#          This is the progam that gets called when the user clicks on 
#          "View Parse" in Nb
# AUTHOR:  Mike McCandless 
# EMAIL:   mike@goldilocks.lcs.mit.edu
# USAGE:   pcng_draw.tcl  "Text annotated in Nb format"
#          Note: always add a space " " after a closing bracket "}"
# 
#         Copyright (c) 1995
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
####################################################################################################

#!draw_parse -f

# DO
#  fix the drawing: allocate per-word amount of space (ie, equal space b/w words)
#  really make this a pcng tool -- put probs in there
#  how about something which generates random phrases for a non-terminal (say, 10 or 100) in a window ?


# set PARSE_FONT -adobe-times-medium-r-normal--18-180-*-*-*-*-*-*
# set TERM_FONT -adobe-courier-bold-r-*-*-*-190-*-*-*-*-*-*
# 
# set TERM_FONT -adobe-helvetica-bold-r-normal--18-180-75-75-p-103-iso8859-1
set TERM_FONT fixed
set TERM_COLOR Red

set TERM_CHAR_WIDTH 0.32
set TERM_CHAR_HEIGHT 0.55

# set NONTERM_FONT -adobe-courier-bold-r-*-*-*-150-*-*-*-*-*-*
set SMALLFONT    fixed
set NONTERM_FONT fixed
# set NONTERM_FONT lucidasanstypewriter-bold-18
set NONTERM_COLOR black
set NONTERM_CHAR_WIDTH 0.5
set NONTERM_CHAR_HEIGHT 0.5

# Name the program window
wm title . "(N)\[b\] View Parse"
wm iconname . "(N)\[b\] View Parse"

# Draw a box of width w and height h centered at x, y
proc draw_centered_box {c x y w h} {
  set x_left [expr $x - $w/2.0]
  set x_right [expr $x + $w/2.0]
  set y_top [expr $y - $h/2.0]
  set y_bot [expr $y + $h/2.0]
  $c create rectangle ${x_left}c ${y_top}c ${x_right}c ${y_bot}c
}


###############################
# START of the parse tcl code #
###############################


# Draw parse on canvas c, with the lower left corner of the parse at the location [x, y] (in cm)

proc parse_draw {c parse x y name} {

    global canvasTag 

    # a simple check to locate the origin :)
    #  $c create oval -100 -100 100 100 -fill black -tags ""foo"
    #  $c delete "foo"
    
    set y_inc 1.75
    set x_inc 2.5
    
    # This is the space required by the parse tree
    set p_height [expr [parse_max_depth $parse] * $y_inc]
    set p_width [expr (1 + [parse_count_terminals $parse]) * $x_inc]
    
    set num_term [expr (1 + [parse_count_terminals $parse])]
    set depth [parse_max_depth $parse]
    
    #  puts "parse: $p_width X $p_height, $num_term terminals, $depth depth";
    
    set foo [$c configure -width]
    #  puts "canvas width: $foo"
    
    set foo [$c configure -height]
    #  puts "canvas height: $foo"
    
    
    # This is the current size of the canvas
    set c_width [lindex [$c configure -width] 3]
    set c_height [lindex [$c configure -height] 3]
    
    
    # Take away the "c" for centimeters
    set c_width [string trimright $c_width "c"]
    set c_height [string trimright $c_height "c"]
    
    #  puts "canvas: $c_width X $c_height, loc: $x,$y";
    
    # this is the region within which we will be drawing -- so set the scroll region
    set x_left [expr $x - 1.0]
    set y_top [expr $y - $p_height - 1.0]
    set x_right [expr $x + $p_width + 1.0]
    set y_bot [expr $y + 1.0]
    
    #  puts "scrollregion: ${x_left}c ${y_top}c ${x_right}c ${y_bot}c"
    $c configure -scrollregion "${x_left}c ${y_top}c ${x_right}c ${y_bot}c"
    
    # This makes it easier later on to compute coordinates
    set x [expr $x + ($x_inc/2.0)]
    set y [expr $y - ($y_inc/2.0)]

    # Draw first the lines...
    set term_upto 0
    parse_r_draw $c $parse $x $y $x_inc $y_inc 0 term_upto ignore 0 0 $name
    
    # ...then the text.
    set term_upto 0
    parse_r_draw $c $parse $x $y $x_inc $y_inc 0 term_upto ignore 0 1 $name
}

proc parse_clear {c name} {
  $c delete $name
}

# This procedure recursively calls itself to draw the actual parse.  X and Y is the
# location of the bottom left corner of the parse, x_inc and y_inc are how many units
# to allocate for each level, and demand_nt is a flag to differentiate the first-level
# call from all subsequent recursions (it's 0 at the first level).  The parameters depth
# and term_upto are used to keep track of where we are currently located in the parse.
# The procedure returns the depth from the bottom of the parse.  Parent_loc is a
# call-by-reference variable to return the location of the parent NT.  If draw_which
# is 0, this function draws the lines.  Otherwise, it draws the text.

proc parse_r_draw {c parse x y x_inc y_inc depth term_upto parent_loc demand_nt draw_which name} {
    global TERM_FONT NONTERM_FONT TERM_COLOR NONTERM_COLOR
    global TERM_CHAR_WIDTH TERM_CHAR_HEIGHT
    global NONTERM_CHAR_WIDTH NONTERM_CHAR_HEIGHT
    global canvasTag

    upvar $term_upto r_term_upto
    upvar $parent_loc r_parent_loc
    
    set first $demand_nt
    set max_depth 0
    set start_term $r_term_upto
    
    set child_locs [list]
    
    
    # This loop goes through all the elements at this level in the parse, recursing
    # as necessary.  As it does this, it collects the locations of all the children
    # in a big list, and at the end filters through this list drawing in the lines.
    
    foreach elem $parse {
    if {$first} {
        set first 0
        continue;
    }
    
    if {[llength $elem] > 1} {
        set depth [parse_r_draw $c $elem $x $y $x_inc $y_inc [expr $depth+1] r_term_upto loc 1 $draw_which $name]
        if {$depth > $max_depth} {
        set max_depth $depth
        }
        
        lappend child_locs $loc
    } else {
        set x_loc [expr $x + $x_inc * $r_term_upto]
        set y_loc $y
        
        lappend child_locs [list $x_loc [expr $y_loc - 0.3]]
        
        if {$draw_which == 1} {
        set new [$c create text ${x_loc}c ${y_loc}c -text $elem -font $TERM_FONT -fill $TERM_COLOR -tags terminal.$name]

        #   set len [string length $elem]
        #   draw_centered_box $c ${x_loc} ${y_loc} [expr $len * $TERM_CHAR_WIDTH] $TERM_CHAR_HEIGHT
        }
        
        incr r_term_upto
        
        #      puts "drawing $elem at loc $x_loc, $y_loc: term $r_term_upto"
    }
    }
    
    set end_term $r_term_upto
    
    

    #  puts "children: $child_locs, start $start_term end $end_term"
    
    # Now, draw lines from the parent NT down to all the children, if appropriate
    if {$demand_nt} {
    set nt [lindex $parse 0]
    
    set x_loc [expr $x + $x_inc * ($start_term + $end_term - 1) / 2.0]
    set y_loc [expr $y - $y_inc * ($max_depth + 1)]
    
    set r_parent_loc [list $x_loc [expr $y_loc-0.4]]
    
    if {$draw_which == 1} {
        set new [$c create text ${x_loc}c ${y_loc}c -text $nt -font $NONTERM_FONT -fill $NONTERM_COLOR -tags node.$nt]
        $c bind node.$nt <Double-Button-1> {CanvasHandler %x %y %W}   
        set canvasTag($new) $nt
    }
    
    set y_loc [expr $y_loc + 0.35]
    
    # puts "nt: $nt elem: $elem draw_which: $draw_which demand_nt: $demand_nt depth: $depth parse: $parse"
    
    set child_loc1 [lindex $child_locs 0]
    set child_loc2 [lindex $child_locs [expr [llength $child_locs] - 1]]
    set reduced_list [list $child_loc1 $child_loc2 ]
    
    # foreach child_loc $child_locs 
    
    foreach child_loc $reduced_list {
        
        set child_x_loc [lindex $child_loc 0]
        set child_y_loc [lindex $child_loc 1]
        
        
        
        if {$draw_which == 0} {
        # puts "line from $x_loc, $y_loc to $child_x_loc, $child_y_loc"
        $c create line ${x_loc}c ${y_loc}c ${child_x_loc}c ${child_y_loc}c -width 1 -tags $name -width 2
        }
    }
    }
    
    return [expr $max_depth+1]
}




# This calculates the maximum depth of the parse

proc parse_max_depth {parse} {
  set max_depth 0

  foreach elem $parse {
    if {[llength $elem] > 1} {
      set new_depth [parse_max_depth $elem]
      if {$new_depth > $max_depth} {
    set max_depth $new_depth
      }
    }
  }

  return [expr $max_depth + 1]
}

# This function recurses down to the single elements, counting their number
proc parse_count_terminals {parse} {

  set width 0

  foreach elem $parse {
    if {[llength $elem] > 1} {
      set width_inc [parse_count_terminals $elem]
      incr width $width_inc
    } else {
      incr width
    }
  }

  incr width -1

  return $width
}


#############################
# END of the parse tcl code #
#############################


# This is my code to set up a canvas and scroll bars and buttons, etc, to loop through pcng
# sentences

# This function reads in another sentence from the file and parses it, if necessary, and
# then draws it.

proc CanvasHandler {x y w} {
    global tagUnderCursor canvasTag idTable
    set id [$w find closest [$w canvasx $x] [$w canvasy $y]]
    if {[info exists canvasTag($id)]} {
    if {[info exists idTable($canvasTag($id))]} {
        set tagUnderCursor $idTable($canvasTag($id))
    } else {
        set tagUnderCursor "$canvasTag($id)"
    }
    } else {
    set tagUnderCursor ""
    }
}


proc draw_current_parse {} {
  global parses cfg f sent_index num_sents first_parse_drawn HEIGHT

  if {$first_parse_drawn} {
    parse_clear .c "the_parse"
  }

  if {$sent_index == $num_sents} {
    if {![eof $f]} {
      gets $f line

      # call the C parser
      puts "parsing...";
      set parses($num_sents) [$cfg parse $line]
      puts "done";

      puts "read `$line'"
      puts "  parse $num_sents `$parses($num_sents)'"

      incr num_sents

      if {[eof $f]} {
    .next configure -state disabled
      }
    } else {
      error "shouldn't be executing this"
    }
  }

  parse_draw .c $parses($sent_index) 1 [expr $HEIGHT - 1] "the_parse"
}


# This function adds one to the current sentence index, taking care to disable the
# next button if it is appropriate, and then draws the parse

proc next_sent {} {
  global f sent_index num_sents

  if {$sent_index == 0} {
    .prev configure -state normal
  }
  
  incr sent_index

  if {$sent_index == [expr $num_sents-1] && [eof $f]} {
    .next configure -state disabled
  }

  draw_current_parse
}



# This function decrements the current sentence index, taking care to disable the
# prev button if it is appropriate, and then draws the parse

proc prev_sent {} {
  global f sent_index num_sents

  if {$sent_index == [expr $num_sents-1] && [eof $f]} {
    .next configure -state normal
  }

  incr sent_index -1

  if {$sent_index == 0} {
    .prev configure -state disabled
  }

  draw_current_parse
}



if {$argc != 1} {
    puts "Usage: pcng_draw.tcl <Text annotated with Nb tags>"
    exit
}

# for the buttons/labels only
set FONT lucidasanstypewriter-bold-18
# set FONT -adobe-courier-bold-r-*-*-*-190-*-*-*-*-*-*

set WIDTH 20
set HEIGHT 16

set WIDTH_PIX [expr $WIDTH * 35]
set HEIGHT_PIX [expr $HEIGHT * 35]

# Set up a canvas

# top holds the canvas and y scrollbar
frame .top

# mid holds the x scrollbar and spacer
frame .mid

# bot holds the control and status
frame .bot

scrollbar .yscroll -command ".c yview" -relief sunken -bg Black -orient vertical -width 15
scrollbar .xscroll -command ".c xview" -relief sunken -bg Black -orient horizontal -width 15

set pad [expr [.yscroll cget -width] + 2* \
         ([.yscroll cget -bd] + \
          [.yscroll cget -highlightthickness])]

frame .space -width $pad -height $pad
frame .bot.space -width $pad -height $pad

# zero width and height so that when the user resizes, the canvas is willing to be
# reduced arbitrarily small (so that the scroll bars and buttons don't disappear)

canvas .c -width 0 -height 0 \
    -yscrollcommand ".yscroll set" \
    -xscrollcommand ".xscroll set" \
    -relief sunken


button .next -font $FONT -text Next -command next_sent
button .prev -font $FONT -text Prev -command prev_sent

pack .top -side top -fill both -expand 1
pack .mid -side top -fill x
pack .bot -side bottom -fill x 

set tagUnderCursor "" 

label  .bot.label -font $NONTERM_FONT -textvariable tagUnderCursor 
button .bot.quit  -font $NONTERM_FONT -command exit -text Dismiss -padx 20 

pack .c -side left -in .top -expand 1 -fill both
pack .yscroll -side left -in .top -fill y
pack .xscroll -side left -in .mid -expand 1 -fill x
pack .space -side right -in .mid
pack .bot.space -side right
pack .bot.quit  -side right  
pack .bot.label -side left


.prev configure -state disabled
.next configure -state disabled

# allow the user to resize
wm minsize . 50 50
wm maxsize . 10000 10000
wm geometry . "${WIDTH_PIX}x${HEIGHT_PIX}"

set parse "foo {WHAT_S what+s} the {FIRST last} {FLT_CLASS_PHRASE {_FLT_CLASS_PHRASE {_FLT_DIR nonstop}}} flight {FROM_CITY from {CITY san francisco}} {TO_CITY to {CITY boston}}"

# puts [format "%s: depth %d, nterm: %d" $parse [parse_max_depth $parse] [parse_count_terminals $parse]]

# set bracketedList [split [lindex $argv 0]]

# Clean-up input argument? Assume input comes from Nb.tcl
set line [lindex $argv 0]
regsub -all {(\[\]\{\}\$\+)} $line " " line
set state          "nil"
set inputLine      $line
set outputString   ""
set bracketCount   0
set openTags       [list]

switch -regexp $line {
    {<.+>}      { set state "isTaggedText" }
    {[a-zA-Z]+} { set state "isText" }
}
if { $state == "isText" } {
    set bracketedList [split $line]
} else {
    while {$line != ""} {
    if {[regexp {^<([-_a-zA-Z0-9]+)=\"([^<>\"]+)\">(.*)} $line match openTag idTag restLine]} {
        # <openTag="idTag">restLine idTag cannot contain < > "
        # puts "openIdTag: <$openTag=\"$idTag\"> restLine: $restLine"
        set line                 $restLine
        set idTable($openTag)    "$openTag $idTag"
        regexp {^([-a-zA-Z0-9]+)\_([0-9]+)$} $openTag match head number
        set concatString " \{ $openTag "
        incr bracketCount
        set outputString $outputString$concatString
        lappend openTags $openTag
    } elseif {[regexp {^<([-_a-zA-Z0-9]+)>(.*)} $line match openTag restLine]} {
        # <openTag>restLine
        # puts "simpleOpenTag: <$openTag> restLine: $restLine"
        set line                 $restLine
        set idTable($openTag)    "$openTag"
        regexp {^([-a-zA-Z0-9]+)\_([0-9]+)$} $openTag match head number
        set concatString " \{ $openTag "
        incr bracketCount
        set outputString $outputString$concatString
        lappend openTags $openTag
    } elseif {[regexp {^</([-_a-zA-Z0-9]+)>(.*)} $line match closeTag restLine]} {
        # </closeTag>restLine           
        # puts "closeTag: </$closeTag> restLine: $restLine"
        set concatString " \} "
        set outputString $outputString$concatString
        set bracketCount [expr $bracketCount - 1]
        if {[lsearch -exact $openTags $closeTag] == -1} {
        set concatString " \{ $closeTag ... "
        set outputString $concatString$outputString
        incr bracketCount
        }
        set     line  $restLine
    } elseif {[regexp {^([^<]+)(<.+)} $line match someText restLine]} {
        # someText<
        # puts "someText: $someText restLine: $restLine"
        set outputString $outputString$someText
        set line $restLine
    } else {
        # default
        set outputString $outputString$line
        # puts "default: $line"
        set line ""
    }
    }
}


# Balance close brackets:
while {$bracketCount > 0} {
    set concatString " ... \} "
    set outputString $outputString$concatString
    set bracketCount [expr $bracketCount - 1]
}
# Balance open brackets:
while {$bracketCount < 0} {
    set concatString "\{ X ... "
    set outputString $concatString$outputString
    incr bracketCount
}   

set bracketedList $outputString
       
regsub -all {[\"]+} $bracketedList " " cleanString

regexp {^[ ]*([^ ]+)} $cleanString match firstWord

# Make a title string:
regsub -all {([-A-Za-z0-9]+_[0-9]+ )} $cleanString "" titleString
regsub -all {([{} ]+)} $titleString " " titleString
set titleList [split $titleString]
set sizeTitle [llength $titleList]
set dots ""
if {$sizeTitle > 10} {
    set sizeTitle 10
    set dots "..."
} 
set titleString ""
set space " "
for { set i 0 } { $i < $sizeTitle } { incr i } {
    set word [lindex $titleList $i]
    set titleString $titleString$space$word
}

wm title . "(N)\[b\] View Parse: $titleString$dots"
wm iconname . "(N)\[b\] View Parse: $titleString$dots"

puts "pcng_draw: FirstWord: ($firstWord) Argument: ($cleanString)"

parse_draw .c $cleanString 1 [expr $HEIGHT - 1] testParse

# set cfg [cfg_load [lindex $argv 0]]

# set f [open [lindex $argv 1]]

set num_sents 0
set sent_index 0

set first_parse_drawn 0

# draw_current_parse

set first_parse_drawn 1

