#####################################################################################################
# FILE:    send_mail.tcl
# PURPOSE: Send annotated files by email
# AUTHOR:  Giovanni Flammia
#
#          (N)[b] Copyright (c) 1995
#          Spoken Language Systems Group
#          Laboratory for Computer Science
#          Massachusetts Institute of Technology
#####################################################################################################


# returns "Success" if success, else return error message.
proc send_mail {files toaddress mailserver} {
  
  global annotatorName 
  
  set success 0
  
  # open client socket connection to mail server on port 23
  if [catch {set s [socket $mailserver 23]}] {
    return "Failure to open socket connection to mail server: $mailserver"
  } 
  fconfigure $s -buffering line
  if {![eof $s]} {
    
    set senderhost "nb-annotator.lcs.mit.edu"
    if [catch {set senderhost [info hostname]}] {
      set senderhost "nb-annotator.lcs.mit.edu"
    }
  
    set sendername $annotatorName
    if {![regexp {[a-zA-Z]+} $sendername match]} {
      set sendername "one-subject"
    }
    
    puts $s "HELO $senderhost"
    puts $s "MAIL FROM: $sendername"
    puts $s "RCPT TO: $toaddress"
    puts $s "DATA"
    flush $s
    
    foreach filename $files {
      if {[file readable $filename]} {
        if {![eof $s]} {
          set file [open $filename r]
          while {[gets $file line] >= 0} {
            if {[regexp {[^ ]+} $line match]} 
            puts $s "$line"
            if {[eof $s]} {
              return "Connection closed prematurely by mail server: $mailserver."
            }
          }
          close $file
          flush $s
        } else {
          return "Connection closed prematurely by mail server: $mailserver."
        }
      } else {
        return "Cannot find annotated file: $filename."
      }
    }
    
    if {![eof $s]} {
      puts $s "."
      puts $s "QUIT"
      flush $s
      close $s
      set success 1
    } else {
      return "Connection closed prematurely by mail server: $mailserver."
    }
  }
  
  if {$success == 1} {
    return "Success!"
  } else {
    return "Failure!"
  }

}
