#!/usr/local/bin/wish40

#####################################################################
# FILE:    view_annotation.tcl
# PURPOSE: Display a colored annotation in a text widget from input tagged file
#          This is the tcl/Tk procedure that is called when the user clicks on
#          "View Annotation" in Nb, or "View Other"
# AUTHOR:  Giovanni Flammia 
#
#         (N)[b] Copyright (c) 1995
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
#####################################################################

puts "Loading: view_annotation.tcl..."


proc UniqueStrings { alist } {
    set result " "
    set spaces " "
    foreach elem $alist {
	if {[regexp " $elem " $result match value] == 1} {
		set result $result
	} else {
	    set result $result$elem$spaces
	}
    }
    return $result
}


# List all the current idTable(Tags) under the cursor:
proc ViewTagsUnderCursor {} {
    global idTable viewUnderCursor viewArea
    set listId ""
    set spaces " "
    foreach oneTag [$viewArea.text tag names "current"] {
	if {[info exists idTable($oneTag)]} {
	    # if {[regexp {( $idTable($oneTag))} $listId match value] == 1} {
	        # set listId $listId 
	    # } else {
	    set listId $listId$spaces$idTable($oneTag)
	    # }
	} else {
	    set listId $listId$spaces$oneTag
	}
    }
    set viewUnderCursor $listId
}




# Hold text and vertical scrollbar
frame .view.top -bg $bg1 

# Hold horizontal scrollbar and padx
frame .view.mid -bg $bg1   

# Hold dismiss and other buttons
frame .view.bot -bg $bg1

# Main window text widget: by default you can't write on it:
set viewArea .view.top
text      $viewArea.text   -relief sunken -font $font1 -bg Black -fg $text_fg \
    -yscrollcommand "$viewArea.yscroll set" \
    -xscrollcommand ".view.mid.xscroll set" \
    -exportselection 1 \
    -state normal \
    -height $annotated_text_lines -wrap none -setgrid 1 
scrollbar $viewArea.yscroll  -orient vertical -command "$viewArea.text yview" -relief raised -bg $scroll_bg
scrollbar .view.mid.xscroll  -orient horizontal -command "$viewArea.text xview" -relief raised -bg $scroll_bg

bind $viewArea.text <Motion> {
    set viewUnderCursor ""
    ViewTagsUnderCursor
}


frame .outline.1 -bg $bg1
frame .outline.2 -bg $bg1
frame .outline.3 -bg $bg1

set outlineLabel "Outline Label"

text .outline.1.text -relief sunken -font $font1 -bg Black -fg $text_fg \
    -exportselection 1 \
    -yscrollcommand ".outline.1.yscroll set" \
    -xscrollcommand ".outline.2.xscroll set" \
    -state normal \
    -height $annotated_text_lines -wrap none -setgrid 1 
scrollbar .outline.1.yscroll  -orient vertical -command ".outline.1.text yview" -relief raised -bg $scroll_bg
scrollbar .outline.2.xscroll  -orient horizontal -command ".outline.1.text xview" -relief raised -bg $scroll_bg

button .outline.3.dismiss -text "Dismiss" -font $font1 -command "wm withdraw .outline" -relief raised -bg $bg1 -fg $fg1 -padx 20 

# create padding based on size of scrollbars (p. 193 of Welch's book)
set pad [expr [$viewArea.yscroll cget -width] + 2* \
	     ([$viewArea.yscroll cget -bd] + \
		  [$viewArea.yscroll cget -highlightthickness])]
frame .view.mid.pad -width $pad -height $pad -bg $bg1
frame .outline.2.pad -width $pad -height $pad -bg $bg1
frame .outline.3.pad -width $pad -height $pad -bg $bg1

button .view.bot.dismiss -text "Dismiss" -font $font1 -command "wm withdraw .view" -relief raised -bg $bg1 -fg $fg1 -padx 20 
frame .view.bot.pad -width $pad -height $pad -bg $bg1
label .view.bot.label -textvariable viewUnderCursor -font $font1 -fg Black -bg $bg1 -padx 20

set viewUnderCursor "View Tags Under Cursor"

# Arrange everything:
pack .view.bot -side bottom -fill x 
pack .view.bot.label -side left
pack .view.bot.pad -side right
pack .view.bot.dismiss -side right
pack .view.mid -side bottom -fill x
pack .view.mid.pad -side right
pack .view.mid.xscroll -side bottom -fill x
pack $viewArea -side top -expand true -fill both
pack $viewArea.yscroll -side right -fill y
pack $viewArea.text -side left -fill both -expand true

pack .outline.1 -side top -fill both -expand true
pack .outline.1.yscroll -side right -fill y 
pack .outline.1.text -side left -fill both -expand true

pack .outline.2 -fill x -expand false
pack .outline.2.pad -side right
pack .outline.2.xscroll -side left -expand true -fill x

pack .outline.3 -fill x -expand false
pack .outline.3.pad -side right
pack .outline.3.dismiss -side right

proc view_annotation {} {

    global sents num_sents textArea viewArea .outline
    global beginAtRow endAtRow cacheFrom cacheTo currentTags currentIndex option 
    global insertIndex viewUnderCursor text_fg text_bg 
    global whiteTags choicesArea fileChanged fileExists output_file_name input_file_name
    global formatTable choiceTable endAtIndex textSelection rowSelection
    global segmentColor colorIndex idTable colorForId colorTable textArea fieldTable choiceTable
    global beginAtIndex endAtIndex whiteTags tagUnderCursor 
    global rangeTable whiteTags

    if {![info exists num_sents]} {
	WarningMessage "I Cant Find An Annotation To Display. Load File?"
	return
    }

    if {$num_sents < 1} {
	WarningMessage "I Cant Find An Annotation To Display. Load File?"
	return
    }

    
    if {[info exists $output_file_name]} {
	set file_name $output_file_name
    } else {
	set file_name $input_file_name
    }

  
    set colorIndex 0

    # Display text:
    $viewArea.text configure -state normal
    $viewArea.text delete 1.0 end
    # Display outline:
    set maxwidth 0
    .outline.1.text configure -state normal
    .outline.1.text delete 1.0 end
    .outline.1.text insert end [format " \n"]
    
    wm deiconify .view
    # wm deiconify .outline

    
    set maxDepth 0
    set oldRow 0
    set outlineRow 0
    set spaces(0) ""
    set realspaces(0) ""
    set tags(0) [list TopLevel]
    set sents(0) ""

    set last [$textArea.text index "end - 1 chars"]
    regexp {([0-9]+)\.([0-9]+)} $last match line char
    set num_sents [expr $line + 1]

    for {set row 1} {$row < $num_sents} {incr row} {
	set sents($row) [$textArea.text get $row.0 [$textArea.text index "$row.0 lineend"]]
	set tags($row) [list]
	set spaces($row) ""
	set realspaces($row) ""
	set beginAtRow($row) [list]
	set endAtRow($row) [list]
    }

    wm title .view "Nb View Annotation: [file tail $file_name]"
    wm iconname .view "Nb View Annotation: [file tail $file_name]"
    wm title .outline "Nb Annotation Summary: [file tail $file_name]"
    wm iconname .outline "Nb Annotation Summary: [file tail $file_name]"
    update idletasks

    set tabs    "     "
    set tabSize [string length $tabs]
    set listOfViewTags [list]

    foreach oneTag $currentTags {
      regexp {([a-zA-Z0-9]+)\_([0-9]+)} $oneTag match head number
      if {[lsearch -exact $whiteTags $head] == -1} {
	if {[info exists choiceTable(SegmentTags)]} {
	  if {[lsearch -exact $choiceTable(SegmentTags) $head] != -1} {
	    set fromIndex [lindex $rangeTable($oneTag) 0]
	    set toIndex   [lindex $rangeTable($oneTag) 1]
	    regexp {^([0-9]+)\.([0-9]+)} $fromIndex match fromRow fromCol
	    regexp {^([0-9]+)\.([0-9]+)} $toIndex match toRow toCol
	    set plist [list]
	    if {[info exists beginAtRow($fromRow)]} {
	      set plist $beginAtRow($fromRow)
	    }
	    set beginAtRow($fromRow) [InsertInPriorityList $oneTag $plist]
	    set plist [list]
	    if {[info exists endAtRow($toRow)]} {
	      set plist $endAtRow($toRow)
	    }
	    set endAtRow($toRow) [InsertInPriorityList $oneTag $plist]
	    set viewUnderCursor "Reading: $oneTag"
	    update idletasks
	  }
	}
      }
    }

    set viewUnderCursor "Updating Annotation Display..."
    update idletasks
    
    for {set row 1} {$row < $num_sents} {incr row ; incr oldRow} {
	foreach oldTextTag $tags($oldRow) {
	    if {[info exists endAtRow($oldRow)]} {
		if {[lsearch -exact $endAtRow($oldRow) $oldTextTag] != -1} {
		} else {
		    lappend tags($row) $oldTextTag
		}
	    } else {
		lappend tags($row) $oldTextTag
	    } 
	}
	if {[info exists beginAtRow($row)]} {
	    set TextTagBuffer  $tags($row) 
	    set tags($row) [concat $tags($row) $beginAtRow($row)]
	    foreach seg $beginAtRow($row) {
		incr outlineRow
		set  outlineText($outlineRow) $idTable($seg)
		set  outlineSeg($outlineRow)  [concat $TextTagBuffer $seg]
		set  TextTagBuffer [concat $TextTagBuffer $seg]
	    }
	}
	if {[llength $tags($row)] > $maxDepth} {
	    set maxDepth [llength $tags($row)]
	}

    }


    for {set row 1} {$row < $num_sents} {incr row} {

	set realTags($row) $tags($row)
	while {[llength $tags($row)] < $maxDepth} {
	    lappend tags($row) [lindex $tags($row) [expr [llength $tags($row)] - 1]]
	}

	set col 0
	set spaces($row) ""
	set realspaces($row) ""
	foreach TextTag $tags($row) {
	    set spaces($row) $spaces($row)$tabs
	}
	foreach TextTag $realTags($row) {
	    set realspaces($row) $realspaces($row)$tabs
	}
	set offset($row) [expr [string length $spaces($row)] + [string length $realspaces($row)]]
	set offset($row) [expr $offset($row) + 2]
	$viewArea.text insert end [format "%s %s\n" $spaces($row) $sents($row)]
	# $viewArea.text see $row.0
	if {[expr $row % 2] == 0} {
	    update idletasks
	}

    }

    
    $viewArea.text configure -state disabled

    set doOutline 0
    if {$doOutline == 1} {

      set viewUnderCursor "Updating Summary Display..."
      update idletasks
      
      set index 1
      while {$index <= $outlineRow } {
	set outlineSpaces($index) ""
	set outlineRealSpaces($index) ""
	for {set i 1} {$i < [llength $outlineSeg($index)]} {incr i} {
          set outlineRealSpaces($index) $outlineRealSpaces($index)$tabs
	}
	while {[llength $outlineSeg($index)] < $maxDepth} {
          lappend outlineSeg($index) [lindex $outlineSeg($index) [expr [llength $outlineSeg($index)] - 1]]
	}
	foreach seg $outlineSeg($index) {
          set from [string length $outlineSpaces($index)]
          set outlineSpaces($index) $outlineSpaces($index)$tabs
          set to [string length $outlineSpaces($index)]
          set fromIndex [expr $index + 1].$from
          set toIndex [expr $index + 1].$to
          .outline.1.text tag add $TextTag $fromIndex $toIndex
          .outline.1.text tag configure $TextTag -foreground Black -background $colorTable($seg)
	}
	set str [format "%s %s %s \n" $outlineSpaces($index) $outlineRealSpaces($index)  $outlineText($index)]
	if {[string length $str] > $maxwidth} {
          set maxwidth [string length $str]
	}
	.outline.1.text insert end $str
	incr index
      }
      
      #  .outline.1.text insert end [format " \n"]
      
    }
    
    set viewUnderCursor "Adding Colors For Tags..."
    update idletasks
    
    # Add colors:
    for {set row 1} {$row < $num_sents} {incr row} {
      set col 0
      foreach TextTag $tags($row) {
        set fromIndex "$row.$col"
        set col [expr [expr $col + $tabSize] - 1]
        set toIndex [$viewArea.text index "$row.$col + 1 chars"]
        incr col
        $viewArea.text tag add $TextTag $fromIndex $toIndex
        $viewArea.text tag configure $TextTag -foreground Black -background $colorTable($TextTag)
        lappend listOfViewTags $TextTag
      }
      set fromIndex "$row.$col"
      set toIndex [$viewArea.text index "$row.$col lineend"]
	if {$colorTable($TextTag) != "Black"} {
          $viewArea.text tag add text.$TextTag $fromIndex $toIndex
          $viewArea.text tag configure text.$TextTag -foreground $colorTable($TextTag)
          set idTable(text.$TextTag) [UniqueStrings [lreplace $realTags($row) 0 0]]
          lappend listOfViewTags text.$TextTag
	}
      #### $viewArea.text see $fromIndex
      if {[expr $row % 2] == 0} {
        update idletasks
      }
    }
    
    if {$doOutline == 1} {
      
      set viewUnderCursor "Adding Colors for Summary..."
      update idletasks
      
      set index 1
      while {$index <= $outlineRow } {
	set outlineSpaces($index) ""
	foreach seg $outlineSeg($index) {
          set from [string length $outlineSpaces($index)]
          set outlineSpaces($index) $outlineSpaces($index)$tabs
          set to [string length $outlineSpaces($index)]
          set fromIndex [expr $index + 1].$from
          set toIndex [expr $index + 1].$to
          .outline.1.text tag add outline.$seg $fromIndex $toIndex
          
          .outline.1.text tag configure outline.$seg -foreground Black -background $colorTable($seg)
	}
	set seg [lindex $outlineSeg($index) [expr [llength $outlineSeg($index)] - 1]]
	set fromIndex $toIndex
	set str [format "%s %s %s \n" $outlineSpaces($index) $outlineRealSpaces($index)  $outlineText($index)]
	set to [expr $to + [string length $str]]
	set toIndex [expr $index + 1].$to
	.outline.1.text tag add text.$seg $fromIndex $toIndex
	.outline.1.text tag configure text.$seg -foreground $colorTable($seg) -background Black
	#### .outline.1.text see $fromIndex
	incr index
	update idletasks
      }
    
      set h [expr $outlineRow + 2]
      if {$h > 25} {
	set h 25
      }
      
      if {$maxwidth > 80} {
	set maxwidth 80
      }
      
      
      .outline.1.text configure -state disabled -height $h -width $maxwidth
    }
    
    set viewUnderCursor "Done."
    update idletasks

}


