#!/usr/local/bin/wish40

#####################################################################
# FILE:    view_color_palette.tcl
# PURPOSE: Display a text window with the list of colors used 
#          by all the current tags that have been annotated
#
# AUTHOR:  Giovanni Flammia 
#
#         (N)[b] Copyright (c) 1995
#         Spoken Language Systems Group
#         Laboratory for Computer Science
#         Massachusetts Institute of Technology
#####################################################################

puts "Loading: view_color_palette.tcl..."


# Hold text and vertical scrollbar
frame .colors.top -bg $bg1 

# Hold horizontal scrollbar and padx
frame .colors.mid -bg $bg1   

# Hold dismiss and other buttons
frame .colors.bot -bg $bg1

# Main window text widget: by default you can't write on it:
set paletteArea .colors.top
text      $paletteArea.text   -relief sunken -font $font1 -bg Black -fg $text_fg \
    -yscrollcommand "$paletteArea.yscroll set" \
    -xscrollcommand ".colors.mid.xscroll set" \
    -exportselection 1 \
    -state normal \
    -cursor arrow \
    -height 16 -width 20 -wrap none -setgrid 1 
scrollbar $paletteArea.yscroll  -orient vertical -command "$paletteArea.text yview" -relief raised -bg $scroll_bg
scrollbar .colors.mid.xscroll  -orient horizontal -command "$paletteArea.text xview" -relief raised -bg $scroll_bg

# create padding based on size of scrollbars (p. 193 of Welch's book)
set pad [expr [$paletteArea.yscroll cget -width] + 2* \
        ([$paletteArea.yscroll cget -bd] + \
        [$paletteArea.yscroll cget -highlightthickness])]
frame .colors.mid.pad -width $pad -height $pad -bg $bg1

button .colors.bot.dismiss -text "Dismiss" -font $font1 -command "wm withdraw .colors" -relief raised -bg $bg1 -fg $fg1 -padx 20 
frame .colors.bot.pad -width $pad -height $pad -bg $bg1
button .colors.bot.update -text "Refresh" -font $font1 -command view_color_palette -relief raised -bg $bg1 -fg $fg1 -padx 20 

wm minsize .colors 20 16

# Arrange everything:
pack .colors.bot -side bottom -fill x 

pack .colors.bot.update -side left
pack .colors.bot.pad -side right
pack .colors.bot.dismiss -side right
pack .colors.mid -side bottom -fill x
pack .colors.mid.pad -side right
pack .colors.mid.xscroll -side bottom -fill x
pack $paletteArea -side top -expand true -fill both
pack $paletteArea.yscroll -side right -fill y
pack $paletteArea.text -side left -fill both -expand true


bind $paletteArea.text <ButtonPress-1> {
    PointToTagsUnderCursor
}


proc PointToTagsUnderCursor {} {
    
    global sents num_sents textArea paletteArea .outline
    global beginAtRow endAtRow cacheFrom cacheTo currentTags currentIndex option 
    global insertIndex viewUnderCursor text_fg text_bg 
    global whiteTags choicesArea fileChanged fileExists output_file_name input_file_name
    global formatTable choiceTable endAtIndex textSelection rowSelection
    global segmentColor colorIndex idTable colorForId colorTable textArea fieldTable choiceTable
    global beginAtIndex endAtIndex whiteTags tagUnderCursor 
    global rangeTable whiteTags 

    if {[$textArea.text tag ranges ViewTags.sel] != ""} {
      $textArea.text tag delete ViewTags.sel
    }

    set tags [$paletteArea.text tag names "current"]
    if {[llength $tags] == 1} {
      if {[regexp {([a-z]+)\.([-a-zA-Z0-9\_]+)} [lindex $tags 0] match dontcare thisTag]} {
         set fromIndex [lindex $rangeTable($thisTag) 0]
         set toIndex   [lindex $rangeTable($thisTag) 1]
         $textArea.text tag add ViewTags.sel $fromIndex $toIndex
         $textArea.text tag configure ViewTags.sel -background LightSeaGreen -foreground yellow
         $textArea.text see $fromIndex
      }
    }
}

proc view_color_palette {} {

  global sents num_sents textArea paletteArea .outline
  global beginAtRow endAtRow cacheFrom cacheTo currentTags currentIndex option 
  global insertIndex viewUnderCursor text_fg text_bg 
  global whiteTags choicesArea fileChanged fileExists output_file_name input_file_name
  global formatTable choiceTable endAtIndex textSelection rowSelection
  global segmentColor colorIndex idTable colorForId colorTable textArea fieldTable choiceTable
  global beginAtIndex endAtIndex whiteTags tagUnderCursor 
  global rangeTable whiteTags

    if {![info exists num_sents]} {
      WarningMessage "There are no colored tags to list. Maybe open file to tag?"
      return
    }
    if {$num_sents < 1} {
      WarningMessage "There are no colored tags to list. Maybe open file to tag?"
      return
    }

    $paletteArea.text configure -state normal
    $paletteArea.text delete 1.0 end
    wm deiconify .colors
    update idletasks
    set maxsize 20
    set tabs    "         "
    set tabSize [string length $tabs]
    set pad [expr $tabSize + 2]
    set padplus [expr $tabSize + 3]
    set endtext 50
    set listOfViewTags [list]
    $paletteArea.text configure -state normal
    $paletteArea.text insert end [format "Click on a color rectangle \nto highlight the corresponding text. \n\n"]
    set listUsedColors [list]
    foreach oneTag $currentTags {
      regexp {([a-zA-Z0-9]+)\_([0-9]+)} $oneTag match head number
      if { [lsearch -exact $whiteTags $head] == -1 } {
         if {[info exists colorTable($oneTag)]} {
           if {[regexp {Segment} $oneTag match]} {
             set temp $idTable($oneTag)
             regsub -all {[_ ]+} $temp " " todisplay
             if {[regexp {SubSegment} $oneTag match]} {
               $paletteArea.text insert end [format " %s %s %s\n" $tabs $tabs $todisplay]
               set startchar [string length [format " %s " $tabs]]
               set endchar   [string length [format " %s %s" $tabs $tabs]]
             } else {
               $paletteArea.text insert end [format " %s %s\n" $tabs $todisplay]
               set startchar 1
               set endchar [string length [format " %s" $tabs]] 
             }
             update idletasks
             regexp {([0-9]+)\.([0-9]+)} [$paletteArea.text index end] match lastrow lastcol
             set lastrow  [expr $lastrow - 2]
             regexp {([0-9]+)\.([0-9]+)} [$paletteArea.text index "$lastrow.0 lineend"] match temp size
             if {$maxsize < $size} {
               set maxsize $size
             }
             set fromSlot [$paletteArea.text index  "$lastrow.$startchar"]
             set toSlot   [$paletteArea.text index  "$lastrow.$endchar"]
             set fromText [$paletteArea.text index  "$lastrow.$endchar + 1 chars"]
             set toText   [$paletteArea.text index  "$lastrow.0 lineend"]
             $paletteArea.text tag add slot.$oneTag $fromSlot $toSlot
             $paletteArea.text tag add label.$oneTag $fromText $toText
             $paletteArea.text tag configure slot.$oneTag -foreground Black -background $colorTable($oneTag)
             $paletteArea.text tag configure label.$oneTag -foreground $colorTable($oneTag) 
             $paletteArea.text insert end "\n"
             lappend listUsedColors $colorTable($oneTag)
           }
         }
      }
   }
   $paletteArea.text see [$paletteArea.text index "1.0"]
   if {$maxsize > 39} {
       set maxsize 39
    }
    $paletteArea.text configure -width [expr $maxsize]
    update idletasks
    $paletteArea.text configure -state disabled
    
}

